@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2023 Taler Systems SA
+  Copyright (C) 2014-2025 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -26,6 +26,8 @@
 #include <taler/taler_mhd_lib.h>
 #include <gnunet/gnunet_mhd_compat.h>
 #include "taler_merchant_bank_lib.h"
+#include <regex.h>
+
 
 /**
  * Shorthand for exit jumps.
@@ -177,11 +179,10 @@ struct TMH_MerchantInstance
   bool deleted;
 
   /**
-   * The authentication settings for this instance
-   * were changed via the command-line. Do not check
-   * against the DB value when updating the auth token.
+   * True if email/sms validation is needed before the
+   * instance can be used.
    */
-  bool auth_override;
+  bool validation_needed;
 };
 
 
@@ -424,14 +425,50 @@ enum TMH_AuthScope
   TMH_AS_READ_ONLY = 1,
 
   /**
-   * /login access to renew the token is OK.
+   * 2 is Reserved. Was refreshable pre v42
    */
-  TMH_AS_REFRESHABLE = 2,
+
+  /**
+   * Order creation and payment status check only
+   */
+  TMH_AS_ORDER_SIMPLE = 3,
+
+  /**
+   * Order creation and inventory locking,
+   * includes #TMH_AS_ORDER_SIMPLE
+   */
+  TMH_AS_ORDER_POS = 4,
+
+  /**
+   * Order creation and refund
+   */
+  TMH_AS_ORDER_MGMT = 5,
+
+  /**
+   * Order full
+   * Includes #TMH_AS_ORDER_POS and #TMH_AS_ORDER_MGMT
+   */
+  TMH_AS_ORDER_FULL = 6,
+
+  /**
+   * Full access is granted to everything.
+   * We want to deprecate and remove this!
+   * Old scope "write"
+   */
+  TMH_AS_ALL = 7 | 1 << 30,
 
   /**
    * Full access is granted to everything.
    */
-  TMH_AS_ALL = 7
+  TMH_AS_SPA = 8,
+
+  /**
+   * /login access to renew the token is OK.
+   * This is actually combined with other scopes
+   * and not (usually) used as a scope itself.
+   */
+  TMH_AS_REFRESHABLE = 1 << 30,
+
 };
 
 
@@ -456,11 +493,9 @@ struct TMH_RequestHandler
   const char *url_prefix;
 
   /**
-   * Required authentication scope for this request.  NONE implies that
-   * #TMH_AS_ALL is required unless this is a #MHD_HTTP_METHOD_GET method, in which
-   * case #TMH_AS_READ_ONLY is sufficient.
+   * Required access permission for this request.
    */
-  enum TMH_AuthScope auth_scope;
+  const char *permission;
 
   /**
    * Does this request include an identifier segment
@@ -607,6 +642,11 @@ struct TMH_HandlerContext
    */
   char *infix;
 
+  /**
+   * Which connection was suspended.
+   */
+  struct MHD_Connection *connection;
+
   /**
    * JSON body that was uploaded, NULL if @e has_body is false.
    */
@@ -684,6 +724,34 @@ extern char *TMH_currency;
  */
 extern char *TMH_base_url;
 
+/**
+ * Name of helper program to send e-mail.
+ */
+extern char *TMH_helper_email;
+
+/**
+ * Name of helper program to send SMS.
+ */
+extern char *TMH_helper_sms;
+
+/**
+ * Space-separated list of allowed payment target types.
+ * "*" for "all" (no restriction).
+ */
+extern char *TMH_allowed_payment_targets;
+
+/**
+ * Regular expression further restricting payment target types.
+ * Can be NULL/empty for no restrictions.
+ */
+extern char *TMH_payment_target_regex;
+
+/**
+ * Compiled regular expression, only valid if
+ * #TMH_payment_target_regex is not NULL!
+ */
+extern regex_t TMH_payment_target_re;
+
 /**
  * Length of the TMH_cspecs array.
  */
@@ -724,10 +792,54 @@ extern struct GNUNET_CONTAINER_MultiHashMap *TMH_by_id_map;
 extern struct GNUNET_TIME_Relative TMH_legal_expiration;
 
 /**
- * Initial authorization token.
+ * Default wire delay for new instances.
  */
-extern char *TMH_default_auth;
+extern struct GNUNET_TIME_Relative TMH_default_wire_transfer_delay;
 
+/**
+ * Default payment delay for new instances.
+ */
+extern struct GNUNET_TIME_Relative TMH_default_pay_delay;
+
+/**
+ * Default refund delay for new instances.
+ */
+extern struct GNUNET_TIME_Relative TMH_default_refund_delay;
+
+/**
+ * #GNUNET_YES if protocol version 19 is strictly enforced.
+ * (Default is #GNUNET_NO)
+ */
+extern int TMH_strict_v19;
+
+/**
+ * #GNUNET_YES if authentication is disabled (For testing only!!).
+ * (Default is #GNUNET_NO)
+ */
+extern int TMH_auth_disabled;
+
+/**
+ * #GNUNET_YES if self-provisioning is enabled.
+ */
+extern int TMH_have_self_provisioning;
+
+/**
+ * Set of TAN channels.
+ */
+enum TEH_TanChannelSet
+{
+  TEH_TCS_NONE = 0,
+  TEH_TCS_SMS = 1,
+  TEH_TCS_EMAIL = 2,
+  TEH_TCS_EMAIL_AND_SMS = 3
+};
+
+
+/**
+ * Which TAN channels are mandatory for self-provisioned
+ * accounts and password resets? Bitmask.
+ */
+extern enum TEH_TanChannelSet TEH_mandatory_tan_channels;
 
 /**
  * Callback that frees an instances removing
@@ -812,14 +924,47 @@ TMH_check_auth (const char *token,
  * Compute a @a hash from @a token hashes for
  * merchant instance authentication.
  *
- * @param token the token to check
+ * @param password the password to check
  * @param[out] salt set to a fresh random salt
  * @param[out] hash set to the hash of @a token under @a salt
  */
 void
-TMH_compute_auth (const char *token,
+TMH_compute_auth (const char *password,
                   struct TALER_MerchantAuthenticationSaltP *salt,
                   struct TALER_MerchantAuthenticationHashP *hash);
 
 
+/**
+ * Check if @a candidate permissions are a subset of @a as permissions
+ *
+ * @param as scope to check against
+ * @param candidate scope to check if its permissions are a subset of @a as permissions.
+ * @return true if it was a subset, false otherwise.
+ */
+bool
+TMH_scope_is_subset (enum TMH_AuthScope as,
+                     enum TMH_AuthScope candidate);
+
+
+/**
+ * Return the TMH_AuthScope corresponding to @a name.
+ *
+ * @param name the name to look for
+ * @return the scope corresponding to the name, or TMH_AS_NONE.
+ */
+enum TMH_AuthScope
+TMH_get_scope_by_name (const char *name);
+
+
+/**
+ * Return the name corresponding to @a scop.
+ *
+ * @param scope the scope to look for
+ * @param[out] refreshable outputs if scope value was refreshable
+ * @return the name corresponding to the scope, or NULL.
+ */
+const char *
+TMH_get_name_by_scope (enum TMH_AuthScope scope,
+                       bool *refreshable);
+
 #endif
