@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2019, 2020, 2021, 2023, 2024 Taler Systems SA
+  (C) 2019, 2020, 2021, 2023, 2024, 2025 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -41,9 +41,9 @@
  *
  * When changing this version, you likely want to also update
  * #MERCHANT_PROTOCOL_CURRENT and #MERCHANT_PROTOCOL_AGE in
- * merchant_api_config.c!
+ * merchant_api_get_config.c!
  */
-#define MERCHANT_PROTOCOL_VERSION "18:0:15"
+#define MERCHANT_PROTOCOL_VERSION "22:0:10"
 
 
 /**
@@ -83,57 +83,64 @@ MH_handler_config (const struct TMH_RequestHandler *rh,
                    struct TMH_HandlerContext *hc)
 {
   static struct MHD_Response *response;
-  static struct GNUNET_TIME_Absolute a;
 
   (void) rh;
   (void) hc;
-  if ( (GNUNET_TIME_absolute_is_past (a)) &&
-       (NULL != response) )
-  {
-    MHD_destroy_response (response);
-    response = NULL;
-  }
   if (NULL == response)
   {
     json_t *specs = json_object ();
     json_t *exchanges = json_array ();
-    struct GNUNET_TIME_Timestamp km;
-    char dat[128];
+    json_t *mtc = json_array ();
 
     GNUNET_assert (NULL != specs);
     GNUNET_assert (NULL != exchanges);
-    a = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_UNIT_DAYS);
-    /* Round up to next full day to ensure the expiration
-       time does not become a fingerprint! */
-    a = GNUNET_TIME_absolute_round_down (a,
-                                         GNUNET_TIME_UNIT_DAYS);
-    a = GNUNET_TIME_absolute_add (a,
-                                  GNUNET_TIME_UNIT_DAYS);
-    /* => /config response stays at most 48h in caches! */
-    km = GNUNET_TIME_absolute_to_timestamp (a);
-    TALER_MHD_get_date_string (km.abs_time,
-                               dat);
+    GNUNET_assert (NULL != mtc);
     TMH_exchange_get_trusted (&add_exchange,
                               exchanges);
+    if (0 != (TEH_TCS_SMS & TEH_mandatory_tan_channels))
+      GNUNET_assert (0 ==
+                     json_array_append_new (mtc,
+                                            json_string ("sms")));
+    if (0 != (TEH_TCS_EMAIL & TEH_mandatory_tan_channels))
+      GNUNET_assert (0 ==
+                     json_array_append_new (mtc,
+                                            json_string ("email")));
     for (unsigned int i = 0; i<TMH_num_cspecs; i++)
     {
       const struct TALER_CurrencySpecification *cspec = &TMH_cspecs[i];
 
       if (TMH_test_exchange_configured_for_currency (cspec->currency))
         GNUNET_assert (0 ==
-                       json_object_set_new (specs,
-                                            cspec->currency,
-                                            TALER_CONFIG_currency_specs_to_json
-                                            (
-                                              cspec)));
+                       json_object_set_new (
+                         specs,
+                         cspec->currency,
+                         TALER_JSON_currency_specs_to_json (
+                           cspec)));
     }
     response = TALER_MHD_MAKE_JSON_PACK (
       GNUNET_JSON_pack_string ("currency",
                                TMH_currency),
+      GNUNET_JSON_pack_string ("payment_target_types",
+                               TMH_allowed_payment_targets),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string ("payment_target_regex",
+                                 TMH_payment_target_regex)),
+      GNUNET_JSON_pack_bool ("have_self_provisioning",
+                             GNUNET_YES ==
+                             TMH_have_self_provisioning),
+#ifdef HAVE_DONAU_DONAU_SERVICE_H
+      GNUNET_JSON_pack_bool ("have_donau",
+                             true),
+#else
+      GNUNET_JSON_pack_bool ("have_donau",
+                             false),
+#endif
       GNUNET_JSON_pack_object_steal ("currencies",
                                      specs),
       GNUNET_JSON_pack_array_steal ("exchanges",
                                     exchanges),
+      GNUNET_JSON_pack_array_steal ("mandatory_tan_channels",
+                                    mtc),
       GNUNET_JSON_pack_string (
         "implementation",
         "urn:net:taler:specs:taler-merchant:c-reference"),
@@ -141,14 +148,6 @@ MH_handler_config (const struct TMH_RequestHandler *rh,
                                "taler-merchant"),
       GNUNET_JSON_pack_string ("version",
                                MERCHANT_PROTOCOL_VERSION));
-    GNUNET_break (MHD_YES ==
-                  MHD_add_response_header (response,
-                                           MHD_HTTP_HEADER_EXPIRES,
-                                           dat));
-    GNUNET_break (MHD_YES ==
-                  MHD_add_response_header (response,
-                                           MHD_HTTP_HEADER_CACHE_CONTROL,
-                                           "public,max-age=21600")); /* 6h */
   }
   return MHD_queue_response (connection,
                              MHD_HTTP_OK,
