@@ -258,6 +258,18 @@ lookup_exchange (const char *exchange_url)
 }
 
 
+bool
+TMH_EXCHANGES_check_trusted (
+  const char *exchange_url)
+{
+  struct TMH_Exchange *exchange = lookup_exchange (exchange_url);
+
+  if (NULL == exchange)
+    return false;
+  return exchange->trusted;
+}
+
+
 /**
  * Check if we have any remaining pending requests for the
  * given @a exchange, and if we have the required data, call
@@ -668,7 +680,7 @@ TMH_EXCHANGES_lookup_wire_fee (
 }
 
 
-enum GNUNET_GenericReturnValue
+enum TMH_ExchangeStatus
 TMH_exchange_check_debit (
   const char *instance_id,
   const struct TMH_Exchange *exchange,
@@ -678,9 +690,13 @@ TMH_exchange_check_debit (
   const struct TALER_EXCHANGE_Keys *keys = exchange->keys;
   bool have_kyc = false;
   bool no_access_token = true;
+  enum TMH_ExchangeStatus retry_ok = 0;
+
+  if (GNUNET_TIME_absolute_is_past (exchange->first_retry))
+    retry_ok = TMH_ES_RETRY_OK;
 
   if (NULL == keys)
-    return GNUNET_SYSERR;
+    return TMH_ES_NO_KEYS | retry_ok;
   if (0 != strcasecmp (keys->currency,
                        max_amount->currency))
   {
@@ -689,7 +705,7 @@ TMH_exchange_check_debit (
                 exchange->url,
                 keys->currency,
                 max_amount->currency);
-    return GNUNET_SYSERR;
+    return TMH_ES_NO_CURR | retry_ok;
   }
   {
     struct TALER_NormalizedPayto np;
@@ -701,10 +717,10 @@ TMH_exchange_check_debit (
                                                            np);
     GNUNET_free (np.normalized_payto);
     if (! account_ok)
-      return GNUNET_NO;
+      return TMH_ES_NO_ACC | retry_ok;
   }
   if (! keys->kyc_enabled)
-    return GNUNET_YES;
+    return TMH_ES_OK | retry_ok;
 
   {
     json_t *jlimits = NULL;
@@ -784,7 +800,7 @@ TMH_exchange_check_debit (
         TALER_amount_min (max_amount,
                           max_amount,
                           &kyc_limit);
-      return GNUNET_YES;
+      return TMH_ES_OK | retry_ok;
     } /* END of if qs > 0, NULL != jlimits */
   }
 
@@ -807,7 +823,7 @@ TMH_exchange_check_debit (
                    TALER_amount_set_zero (
                      max_amount->currency,
                      max_amount));
-    return GNUNET_YES;
+    return TMH_ES_OK | retry_ok;
   }
   /* In any case, abide by hard limits (unless we have custom rules). */
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -834,7 +850,7 @@ TMH_exchange_check_debit (
                    TALER_amount_set_zero (max_amount->currency,
                                           max_amount));
   }
-  return GNUNET_YES;
+  return TMH_ES_OK | retry_ok;
 }
 
 
