@@ -75,6 +75,18 @@ typedef void
 struct TMH_EXCHANGES_KeysOperation;
 
 
+/**
+ * Check if we trust the exchange at @a exchange_url.
+ *
+ * @param exchange_url exchange base url to check
+ * @return true if we trust that exchange (assuming the master
+ *   public key matches)
+ */
+bool
+TMH_EXCHANGES_check_trusted (
+  const char *exchange_url);
+
+
 /**
  * Get /keys of the given @a exchange.
  *
@@ -152,6 +164,23 @@ TMH_EXCHANGES_get_currency (
   const struct TMH_Exchange *exchange);
 
 
+/**
+ * Check if the given operation and amount would
+ * violate any hard limits of the exchange.
+ * Only useful for transaction and refund limits.
+ *
+ * @param keys the keys of the exchange to check limit for
+ * @param operation_type the kind of operation we perform
+ * @param amount the amount we want to transact
+ * @return true if this is allowed
+ */
+bool
+TMH_EXCHANGES_is_below_limit (
+  const struct TALER_EXCHANGE_Keys *keys,
+  enum TALER_KYCLOGIC_KycTriggerEvent operation_type,
+  const struct TALER_Amount *amount);
+
+
 /**
  * Lookup current wire fee by @a exchange_url and @a wire_method.
  *
@@ -174,14 +203,39 @@ TMH_EXCHANGES_lookup_wire_fee (
  * wm. Checks that both @a ex is trusted and that @a ex allows wire transfers
  * into the account given in @a wm.
  *
+ * @param instance_id the instance to check for
  * @param exchange the exchange to check
  * @param wm the wire method to check with
- * @return #GNUNET_OK if such a debit can happen
+ * @param[in,out] max_amount maximum amount we may deposit at this
+ *    exchange; input is an existing maximum, that
+ *    can be lowered by this function due to transaction
+ *    limits and deposit limits of the exchange
+ * @return #TMH_ES_OK if such a debit can happen
+ *         #TMH_ES_NO_ACC if the exchange cannot
+ *            be used due to account restrictions
+ *         #TMH_ES_NO_CURR if the exchange cannot be used
+ *            because it is for a different currency
+ *         #TMH_ES_NO_KEYS if the exchange cannot be used
+ *            because we do not even know its keys
+ *         #TMH_ES_RETRY_OK if the exchange keys request
+ *            could be retried (OR-bit!)
  */
-enum GNUNET_GenericReturnValue
+enum TMH_ExchangeStatus
+{
+  TMH_ES_OK = 0,
+  TMH_ES_NO_ACC = 1,
+  TMH_ES_NO_CURR = 2,
+  TMH_ES_NO_KEYS = 3,
+  TMH_ES_RETRY_OK = 16,
+  TMH_ES_NO_ACC_RETRY_OK = TMH_ES_NO_ACC | TMH_ES_RETRY_OK,
+  TMH_ES_NO_CURR_RETRY_OK = TMH_ES_NO_CURR | TMH_ES_RETRY_OK,
+  TMH_ES_NO_KEYS_RETRY_OK = TMH_ES_NO_KEYS | TMH_ES_RETRY_OK,
+}
 TMH_exchange_check_debit (
+  const char *instance_id,
   const struct TMH_Exchange *exchange,
-  const struct TMH_WireMethod *wm);
+  const struct TMH_WireMethod *wm,
+  struct TALER_Amount *max_amount);
 
 
 /**
@@ -196,4 +250,20 @@ TMH_test_exchange_configured_for_currency (
   const char *currency);
 
 
+/**
+ * Determines the legal limit for a given @a operation_type
+ * at a given @a exchange_url.
+ *
+ * @param exchange_url base URL of the exchange to get
+ *   the refund limit for
+ * @param operation_type type of operation
+ * @param[in,out] amount lowered to the maximum
+ *   allowed at the exchange
+ */
+void
+TMH_EXCHANGES_get_limit (
+  const char *exchange_url,
+  enum TALER_KYCLOGIC_KycTriggerEvent operation_type,
+  struct TALER_Amount *amount);
+
 #endif
