@@ -529,42 +529,124 @@ TMH_setup_wire_account (
 }
 
 
+enum TALER_ErrorCode
+TMH_check_token (const char *token,
+                 const char *instance_id,
+                 enum TMH_AuthScope *as)
+{
+  enum TMH_AuthScope scope;
+  struct GNUNET_TIME_Timestamp expiration;
+  enum GNUNET_DB_QueryStatus qs;
+  struct TALER_MERCHANTDB_LoginTokenP btoken;
+
+  if (NULL == token)
+  {
+    *as = TMH_AS_NONE;
+    return TALER_EC_NONE;
+  }
+  if (0 != strncasecmp (token,
+                        RFC_8959_PREFIX,
+                        strlen (RFC_8959_PREFIX)))
+  {
+    *as = TMH_AS_NONE;
+    return TALER_EC_NONE;
+  }
+  token += strlen (RFC_8959_PREFIX);
+  if (GNUNET_OK !=
+      GNUNET_STRINGS_string_to_data (token,
+                                     strlen (token),
+                                     &btoken,
+                                     sizeof (btoken)))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Given authorization token `%s' is malformed\n",
+                token);
+    GNUNET_break_op (0);
+    return TALER_EC_GENERIC_TOKEN_MALFORMED;
+  }
+  qs = TMH_db->select_login_token (TMH_db->cls,
+                                   instance_id,
+                                   &btoken,
+                                   &expiration,
+                                   (uint32_t*) &scope);
+  if (qs < 0)
+  {
+    GNUNET_break (0);
+    return TALER_EC_GENERIC_DB_FETCH_FAILED;
+  }
+  if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Authorization token `%s' unknown\n",
+                token);
+    return TALER_EC_GENERIC_TOKEN_UNKNOWN;
+  }
+  if (GNUNET_TIME_absolute_is_past (expiration.abs_time))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Authorization token `%s' expired\n",
+                token);
+    return TALER_EC_GENERIC_TOKEN_EXPIRED;
+  }
+  *as = scope;
+  return TALER_EC_NONE;
+}
+
+
 enum GNUNET_GenericReturnValue
 TMH_check_auth_config (struct MHD_Connection *connection,
                        const json_t *jauth,
-                       const char **auth_token)
+                       const char **auth_password)
 {
   bool auth_wellformed = false;
   const char *auth_method = json_string_value (json_object_get (jauth,
                                                                 "method"));
 
-  *auth_token = NULL;
+  *auth_password = NULL;
   if (NULL == auth_method)
   {
     GNUNET_break_op (0);
   }
-  else if (0 == strcmp (auth_method,
-                        "external"))
+  else if ((GNUNET_YES != TMH_strict_v19) &&
+           (0 == strcmp (auth_method,
+                         "external")))
+  {
+    auth_wellformed = true;
+  }
+  else if (GNUNET_YES == TMH_auth_disabled)
   {
     auth_wellformed = true;
   }
   else if (0 == strcmp (auth_method,
                         "token"))
   {
-    *auth_token = json_string_value (json_object_get (jauth,
-                                                      "token"));
-    if (NULL == *auth_token)
+    json_t *pw_value;
+
+    pw_value = json_object_get (jauth,
+                                "password");
+    if (NULL == pw_value)
     {
+      pw_value = json_object_get (jauth,
+                                  "token");
+    }
+    if (NULL == pw_value)
+    {
+      auth_wellformed = false;
       GNUNET_break_op (0);
     }
     else
     {
-      if (0 != strncasecmp (RFC_8959_PREFIX,
-                            *auth_token,
-                            strlen (RFC_8959_PREFIX)))
-        GNUNET_break_op (0);
-      else
+      *auth_password = json_string_value (pw_value);
+      if (NULL != *auth_password)
+      {
+        if (0 == strncasecmp (RFC_8959_PREFIX,
+                              *auth_password,
+                              strlen (RFC_8959_PREFIX)))
+        {
+          *auth_password = *auth_password + strlen (RFC_8959_PREFIX);
+        }
         auth_wellformed = true;
+      }
     }
   }
 
