@@ -383,35 +383,36 @@ generate_success_response (struct AbortContext *ac)
         GNUNET_JSON_pack_string ("type",
                                  "undeposited"));
     }
-    else if (MHD_HTTP_OK != rdi->http_status)
+    else
     {
-      detail = GNUNET_JSON_PACK (
-        GNUNET_JSON_pack_string ("type",
-                                 "failure"),
-        GNUNET_JSON_pack_uint64 ("exchange_status",
-                                 rdi->http_status),
-        GNUNET_JSON_pack_uint64 ("exchange_code",
-                                 (NULL != rdi->exchange_reply)
-                                                          ?
-                                 TALER_JSON_get_error_code (
-                                   rdi->exchange_reply)
-                                                          :
-                                 TALER_EC_GENERIC_INVALID_RESPONSE),
-        GNUNET_JSON_pack_allow_null (
-          GNUNET_JSON_pack_object_incref ("exchange_reply",
-                                          rdi->exchange_reply)));
-    }
-    else if (rdi->found_deposit)
-    {
-      detail = GNUNET_JSON_PACK (
-        GNUNET_JSON_pack_string ("type",
-                                 "success"),
-        GNUNET_JSON_pack_uint64 ("exchange_status",
-                                 rdi->http_status),
-        GNUNET_JSON_pack_data_auto ("exchange_sig",
-                                    &rdi->exchange_sig),
-        GNUNET_JSON_pack_data_auto ("exchange_pub",
-                                    &rdi->exchange_pub));
+      if (MHD_HTTP_OK != rdi->http_status)
+      {
+        detail = GNUNET_JSON_PACK (
+          GNUNET_JSON_pack_string ("type",
+                                   "failure"),
+          GNUNET_JSON_pack_uint64 ("exchange_status",
+                                   rdi->http_status),
+          GNUNET_JSON_pack_uint64 ("exchange_code",
+                                   (NULL != rdi->exchange_reply)
+                                   ? TALER_JSON_get_error_code (
+                                     rdi->exchange_reply)
+                                   : TALER_EC_GENERIC_INVALID_RESPONSE),
+          GNUNET_JSON_pack_allow_null (
+            GNUNET_JSON_pack_object_incref ("exchange_reply",
+                                            rdi->exchange_reply)));
+      }
+      else
+      {
+        detail = GNUNET_JSON_PACK (
+          GNUNET_JSON_pack_string ("type",
+                                   "success"),
+          GNUNET_JSON_pack_uint64 ("exchange_status",
+                                   rdi->http_status),
+          GNUNET_JSON_pack_data_auto ("exchange_sig",
+                                      &rdi->exchange_sig),
+          GNUNET_JSON_pack_data_auto ("exchange_pub",
+                                      &rdi->exchange_pub));
+      }
     }
     GNUNET_assert (0 ==
                    json_array_append_new (refunds,
@@ -796,6 +797,7 @@ begin_transaction (struct AbortContext *ac)
                        &h_contract_terms))
     {
       GNUNET_break_op (0);
+      TMH_db->rollback (TMH_db->cls);
       resume_abort_with_error (ac,
                                MHD_HTTP_FORBIDDEN,
                                TALER_EC_MERCHANT_POST_ORDERS_ID_ABORT_CONTRACT_HASH_MISSMATCH,
@@ -893,10 +895,13 @@ parse_abort (struct MHD_Connection *connection,
   if (0 == ac->coins_cnt)
   {
     GNUNET_break_op (0);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_BAD_REQUEST,
-                                       TALER_EC_MERCHANT_POST_ORDERS_ID_ABORT_COINS_ARRAY_EMPTY,
-                                       "coins");
+    return (MHD_YES ==
+            TALER_MHD_reply_with_error (connection,
+                                        MHD_HTTP_BAD_REQUEST,
+                                        TALER_EC_MERCHANT_POST_ORDERS_ID_ABORT_COINS_ARRAY_EMPTY,
+                                        "coins"))
+      ? GNUNET_NO
+      : GNUNET_SYSERR;
   }
   /* note: 1 coin = 1 deposit confirmation expected */
   ac->pending = ac->coins_cnt;
