@@ -46,6 +46,10 @@ struct UseContext
    */
   struct TALER_MERCHANTDB_TemplateDetails etp;
 
+  /**
+   * True once @e etp was initialized.
+   */
+  bool have_etp;
 };
 
 
@@ -79,7 +83,6 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
   const char *fulfillment_message = NULL;
   struct TALER_Amount amount;
   bool no_amount;
-  json_t *fake_body;
   bool no_summary;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_mark_optional (
@@ -117,6 +120,7 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
     }
   }
 
+  if (! uc->have_etp)
   {
     enum GNUNET_DB_QueryStatus qs;
 
@@ -154,10 +158,11 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
         template_id);
     case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
       /* all good */
+      uc->have_etp = true;
       break;
     } /* End of the switch */
   }
-
+  if (NULL == uc->ihc.request_body)
   {
     /* template */
     const char *tsummary = NULL;
@@ -185,6 +190,7 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
                                       &pay_duration),
       GNUNET_JSON_spec_end ()
     };
+    json_t *fake_body;
 
     {
       enum GNUNET_GenericReturnValue res;
@@ -291,11 +297,11 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
               fulfillment_message))
           ))
       );
+    uc->ihc.request_body = fake_body;
   }
 
-  uc->ihc.request_body = fake_body;
   return TMH_private_post_orders (
-    NULL,                                                  /* not even used */
+    NULL,    /* not even used */
     connection,
     &uc->ihc);
 }
