@@ -83,8 +83,8 @@ TMH_private_get_accounts_ID (const struct TMH_RequestHandler *rh,
       MHD_HTTP_OK,
       GNUNET_JSON_pack_bool ("active",
                              tp.active),
-      GNUNET_JSON_pack_string ("payto_uri",
-                               tp.payto_uri),
+      TALER_JSON_pack_full_payto ("payto_uri",
+                                  tp.payto_uri),
       GNUNET_JSON_pack_data_auto ("h_wire",
                                   &tp.h_wire),
       GNUNET_JSON_pack_data_auto ("salt",
@@ -95,7 +95,7 @@ TMH_private_get_accounts_ID (const struct TMH_RequestHandler *rh,
     /* We do not return the credentials, as they may
        be sensitive */
     json_decref (tp.credit_facade_credentials);
-    GNUNET_free (tp.payto_uri);
+    GNUNET_free (tp.payto_uri.full_payto);
     GNUNET_free (tp.credit_facade_url);
     return ret;
   }
