@@ -30,25 +30,6 @@
 #include <taler/taler_dbevents.h>
 #include <regex.h>
 
-/**
- * We do not re-check an acceptable KYC status for
- * a month, as usually a KYC never expires.
- */
-#define STALE_KYC_TIMEOUT GNUNET_TIME_UNIT_MONTHS
-
-/**
- * How long should clients cache a KYC failure response?
- */
-#define EXPIRATION_KYC_FAILURE GNUNET_TIME_relative_multiply ( \
-          GNUNET_TIME_UNIT_MINUTES, 5)
-
-/**
- * How long should clients cache a KYC success response?
- */
-#define EXPIRATION_KYC_SUCCESS GNUNET_TIME_relative_multiply ( \
-          GNUNET_TIME_UNIT_HOURS, 1)
-
-
 /**
  * Information we keep per /kyc request.
  */
@@ -363,24 +344,10 @@ kyc_context_cleanup (void *cls)
 static void
 resume_kyc_with_response (struct KycContext *kc)
 {
-  char dat[128];
-
   kc->response_code = MHD_HTTP_OK;
   kc->response = TALER_MHD_MAKE_JSON_PACK (
     GNUNET_JSON_pack_array_incref ("kyc_data",
                                    kc->kycs_data));
-  /* KYC failed, cache briefly */
-  TALER_MHD_get_date_string (GNUNET_TIME_relative_to_absolute (
-                               EXPIRATION_KYC_FAILURE),
-                             dat);
-  GNUNET_break (MHD_YES ==
-                MHD_add_response_header (kc->response,
-                                         MHD_HTTP_HEADER_EXPIRES,
-                                         dat));
-  GNUNET_break (MHD_YES ==
-                MHD_add_response_header (kc->response,
-                                         MHD_HTTP_HEADER_CACHE_CONTROL,
-                                         "max-age=300"));
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Resuming /kyc handling as exchange interaction is done (%u)\n",
               MHD_HTTP_OK);
@@ -408,12 +375,14 @@ kyc_change_cb (void *cls,
 {
   struct KycContext *kc = cls;
 
-  GNUNET_assert (GNUNET_YES == kc->suspended);
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Resuming KYC with gateway timeout\n");
-  kc->suspended = GNUNET_NO;
-  MHD_resume_connection (kc->connection);
-  TALER_MHD_daemon_trigger (); /* we resumed, kick MHD */
+  if (GNUNET_YES == kc->suspended)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Resuming KYC with gateway timeout\n");
+    kc->suspended = GNUNET_NO;
+    MHD_resume_connection (kc->connection);
+    TALER_MHD_daemon_trigger (); /* we resumed, kick MHD */
+  }
 }
 
 
@@ -942,6 +911,13 @@ kyc_status_cb (
   struct KycContext *kc = cls;
   struct ExchangeKycRequest *ekr;
 
+  if (! TMH_EXCHANGES_check_trusted (exchange_url))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Skipping exchange `%s': not trusted\n",
+                exchange_url);
+    return;
+  }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "KYC status for `%s' at `%s' is %u/%s/%s/%s\n",
               payto_uri.full_payto,
