@@ -945,6 +945,8 @@ phase_check_repurchase (struct GetOrderRequestContext *gorc)
                               &gorc->contract_amount)),
     GNUNET_JSON_pack_string ("summary",
                              gorc->contract_terms->summary),
+    GNUNET_JSON_pack_timestamp ("pay_deadline",
+                                gorc->contract_terms->pay_deadline),
     GNUNET_JSON_pack_timestamp ("creation_time",
                                 gorc->contract_terms->timestamp));
   GNUNET_free (order_status_url);
