@@ -117,6 +117,8 @@ add_product (void *cls,
     json_array_append_new (
       pa,
       GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_string ("product_name",
+                                 pd->product_name),
         GNUNET_JSON_pack_string ("description",
                                  pd->description),
         GNUNET_JSON_pack_object_incref ("description_i18n",
