@@ -29,6 +29,7 @@
 
 /**
  * PATCH configuration of an existing instance, given its configuration.
+ * This is the handler called using the instance's own authentication.
  *
  * @param rh context of the handler
  * @param connection the MHD connection to handle
@@ -40,4 +41,19 @@ TMH_private_patch_instances_ID (const struct TMH_RequestHandler *rh,
                                 struct MHD_Connection *connection,
                                 struct TMH_HandlerContext *hc);
 
+
+/**
+ * PATCH configuration of an existing instance, given its configuration.
+ * This is the handler called using the default instance's authentication.
+ *
+ * @param rh context of the handler
+ * @param connection the MHD connection to handle
+ * @param[in,out] hc context with further information about the request
+ * @return MHD result code
+ */
+MHD_RESULT
+TMH_private_patch_instances_default_ID (const struct TMH_RequestHandler *rh,
+                                        struct MHD_Connection *connection,
+                                        struct TMH_HandlerContext *hc);
+
 #endif
