@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2023 Taler Systems SA
+  (C) 2014-2023, 2025 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -32,7 +32,7 @@
 /**
  * How often do we retry the simple INSERT database transaction?
  */
-#define MAX_RETRIES 3
+#define MAX_RETRIES 5
 
 
 MHD_RESULT
@@ -40,7 +40,7 @@ TMH_private_post_transfers (const struct TMH_RequestHandler *rh,
                             struct MHD_Connection *connection,
                             struct TMH_HandlerContext *hc)
 {
-  const char *payto_uri;
+  struct TALER_FullPayto payto_uri;
   const char *exchange_url;
   struct TALER_WireTransferIdentifierRawP wtid;
   struct TALER_Amount amount;
@@ -49,8 +49,8 @@ TMH_private_post_transfers (const struct TMH_RequestHandler *rh,
                                 &amount),
     GNUNET_JSON_spec_fixed_auto ("wtid",
                                  &wtid),
-    TALER_JSON_spec_payto_uri ("payto_uri",
-                               &payto_uri),
+    TALER_JSON_spec_full_payto_uri ("payto_uri",
+                                    &payto_uri),
     TALER_JSON_spec_web_url ("exchange_url",
                              &exchange_url),
     GNUNET_JSON_spec_end ()
@@ -68,7 +68,7 @@ TMH_private_post_transfers (const struct TMH_RequestHandler *rh,
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "New inbound wire transfer over %s to %s from %s\n",
               TALER_amount2s (&amount),
-              payto_uri,
+              payto_uri.full_payto,
               exchange_url);
 
   /* Check if transfer data is in database, if not, add it. */
@@ -91,13 +91,7 @@ TMH_private_post_transfers (const struct TMH_RequestHandler *rh,
                                   &wtid,
                                   &amount,
                                   payto_uri,
-                                  true /* confirmed! */);
-    if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
-      qs = TMH_db->set_transfer_status_to_confirmed (TMH_db->cls,
-                                                     hc->instance->settings.id,
-                                                     exchange_url,
-                                                     &wtid,
-                                                     &amount);
+                                  0 /* no bank serial known! */);
     switch (qs)
     {
     case GNUNET_DB_STATUS_HARD_ERROR:
@@ -111,13 +105,13 @@ TMH_private_post_transfers (const struct TMH_RequestHandler *rh,
       TMH_db->rollback (TMH_db->cls);
       continue;
     case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
-      /* Could not set to confirmed, must differ by amount! */
+      /* Must mean the bank account is unknown! */
       TMH_db->rollback (TMH_db->cls);
       return TALER_MHD_reply_with_error (
         connection,
         MHD_HTTP_CONFLICT,
         TALER_EC_MERCHANT_PRIVATE_POST_TRANSFERS_CONFLICTING_SUBMISSION,
-        NULL);
+        payto_uri.full_payto);
     case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
       break;
     }
@@ -151,6 +145,7 @@ TMH_private_post_transfers (const struct TMH_RequestHandler *rh,
                   "post-transfer committed successfully\n");
       break;
     }
+    break;
   }
   return TALER_MHD_reply_static (connection,
                                  MHD_HTTP_NO_CONTENT,
