@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2023-2024 Taler Systems SA
+  Copyright (C) 2023-2025 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -19,6 +19,7 @@
  * @author Christian Grothoff
  */
 #include "platform.h"
+#include "microhttpd.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <jansson.h>
 #include <pthread.h>
@@ -152,11 +153,6 @@ struct Inquiry
    */
   struct TALER_WireTransferIdentifierRawP wtid;
 
-  /**
-   * Amount of the wire transfer.
-   */
-  struct TALER_Amount total;
-
   /**
    * Row of the wire transfer in our database.
    */
@@ -281,28 +277,29 @@ launch_inquiries_at_exchange (struct Exchange *e)
  *
  * @param w inquiry to update status for
  * @param next_attempt when should we retry @a w (if ever)
+ * @param http_status HTTP status of the response
  * @param ec error code to use (if any)
- * @param failed failure status (if ultimately failed)
- * @param verified success status (if ultimately successful)
+ * @param last_hint hint delivered with the response (if any, possibly NULL)
+ * @param needs_retry true if we should try the HTTP request again
  */
 static void
 update_transaction_status (const struct Inquiry *w,
                            struct GNUNET_TIME_Absolute next_attempt,
+                           unsigned int http_status,
                            enum TALER_ErrorCode ec,
-                           bool failed,
-                           bool verified)
+                           const char *last_hint,
+                           bool needs_retry)
 {
   enum GNUNET_DB_QueryStatus qs;
 
-  if (failed)
-    found_problem = true;
   qs = db_plugin->update_transfer_status (db_plugin->cls,
                                           w->exchange->exchange_url,
                                           &w->wtid,
                                           next_attempt,
+                                          http_status,
                                           ec,
-                                          failed,
-                                          verified);
+                                          last_hint,
+                                          needs_retry);
   if (qs < 0)
   {
     GNUNET_break (0);
@@ -324,9 +321,11 @@ sync_keys (struct Exchange *e)
 {
   enum GNUNET_DB_QueryStatus qs;
   struct TALER_EXCHANGE_Keys *keys;
+  struct GNUNET_TIME_Absolute first_retry;
 
   qs = db_plugin->select_exchange_keys (db_plugin->cls,
                                         e->exchange_url,
+                                        &first_retry,
                                         &keys);
   if (qs < 0)
   {
@@ -722,18 +721,13 @@ wire_transfer_cb (void *cls,
     break;
   case MHD_HTTP_BAD_REQUEST:
   case MHD_HTTP_FORBIDDEN:
-    update_transaction_status (w,
-                               GNUNET_TIME_UNIT_FOREVER_ABS,
-                               TALER_EC_MERCHANT_EXCHANGE_TRANSFERS_HARD_FAILURE,
-                               true,
-                               false);
-    end_inquiry (w);
-    return;
   case MHD_HTTP_NOT_FOUND:
+    found_problem = true;
     update_transaction_status (w,
                                GNUNET_TIME_UNIT_FOREVER_ABS,
-                               TALER_EC_MERCHANT_EXCHANGE_TRANSFERS_FATAL_NOT_FOUND,
-                               true,
+                               tgr->hr.http_status,
+                               tgr->hr.ec,
+                               tgr->hr.hint,
                                false);
     end_inquiry (w);
     return;
@@ -744,12 +738,14 @@ wire_transfer_cb (void *cls,
     update_transaction_status (w,
                                GNUNET_TIME_relative_to_absolute (
                                  e->transfer_delay),
-                               TALER_EC_MERCHANT_EXCHANGE_TRANSFERS_TRANSIENT_FAILURE,
-                               false,
-                               false);
+                               tgr->hr.http_status,
+                               tgr->hr.ec,
+                               tgr->hr.hint,
+                               true);
     end_inquiry (w);
     return;
   default:
+    found_problem = true;
     e->transfer_delay = GNUNET_TIME_STD_BACKOFF (e->transfer_delay);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected HTTP status %u\n",
@@ -757,9 +753,10 @@ wire_transfer_cb (void *cls,
     update_transaction_status (w,
                                GNUNET_TIME_relative_to_absolute (
                                  e->transfer_delay),
-                               TALER_EC_MERCHANT_EXCHANGE_TRANSFERS_TRANSIENT_FAILURE,
-                               false,
-                               false);
+                               tgr->hr.http_status,
+                               tgr->hr.ec,
+                               tgr->hr.hint,
+                               true);
     end_inquiry (w);
     return;
   }
@@ -782,6 +779,11 @@ wire_transfer_cb (void *cls,
       GNUNET_SCHEDULER_shutdown ();
       return;
     }
+    // FIXME: insert_transfer_details has more complex
+    // error possibilities inside, expose them here
+    // and persist them with the transaction status
+    // if they arise (especially no_account, no_exchange, conflict)
+    // -- not sure how no_instance could happen...
     if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -860,9 +862,10 @@ wire_transfer_cb (void *cls,
         ? GNUNET_TIME_UNIT_FOREVER_ABS
         : GNUNET_TIME_relative_to_absolute (
           GNUNET_TIME_UNIT_MINUTES),
+        MHD_HTTP_OK,
         ctc.ec,
-        ctc.failure,
-        false);
+        NULL /* no hint */,
+        ! ctc.failure);
       end_inquiry (w);
       return;
     }
@@ -876,41 +879,33 @@ wire_transfer_cb (void *cls,
     GNUNET_break_op (0);
     update_transaction_status (w,
                                GNUNET_TIME_UNIT_FOREVER_ABS,
+                               MHD_HTTP_OK,
                                TALER_EC_MERCHANT_PRIVATE_POST_TRANSFERS_BAD_WIRE_FEE,
-                               true,
+                               TALER_amount2s (&td->wire_fee),
                                false);
     end_inquiry (w);
     return;
   }
 
-  if ( (GNUNET_OK !=
-        TALER_amount_cmp_currency (&td->total_amount,
-                                   &w->total)) ||
-       (0 !=
-        TALER_amount_cmp (&td->total_amount,
-                          &w->total)) )
   {
-    GNUNET_break_op (0);
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Wire transfer total value was %s\n",
-                TALER_amount2s (&w->total));
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Exchange claimed total value to be %s\n",
-                TALER_amount2s (&td->total_amount));
-    update_transaction_status (w,
-                               GNUNET_TIME_UNIT_FOREVER_ABS,
-                               TALER_EC_MERCHANT_EXCHANGE_TRANSFERS_CONFLICTING_TRANSFERS,
-                               true,
-                               false);
-    end_inquiry (w);
-    return;
+    enum GNUNET_DB_QueryStatus qs;
+
+    qs = db_plugin->finalize_transfer_status (db_plugin->cls,
+                                              w->exchange->exchange_url,
+                                              &w->wtid,
+                                              &td->h_details,
+                                              &td->total_amount,
+                                              &td->wire_fee,
+                                              &td->exchange_pub,
+                                              &td->exchange_sig);
+    if (qs < 0)
+    {
+      GNUNET_break (0);
+      global_ret = EXIT_FAILURE;
+      GNUNET_SCHEDULER_shutdown ();
+      return;
+    }
   }
-  /* set transaction to successful */
-  update_transaction_status (w,
-                             GNUNET_TIME_UNIT_FOREVER_ABS,
-                             TALER_EC_NONE,
-                             false,
-                             true);
   end_inquiry (w);
 }
 
@@ -944,9 +939,10 @@ exchange_request (void *cls)
     update_transaction_status (w,
                                GNUNET_TIME_relative_to_absolute (
                                  e->transfer_delay),
+                               0 /* failed to begin */,
                                TALER_EC_MERCHANT_EXCHANGE_TRANSFERS_TRANSIENT_FAILURE,
-                               false,
-                               false);
+                               "Failed to initiate GET request at exchange",
+                               true);
     end_inquiry (w);
     return;
   }
@@ -954,9 +950,10 @@ exchange_request (void *cls)
   update_transaction_status (w,
                              GNUNET_TIME_relative_to_absolute (
                                GNUNET_TIME_UNIT_MINUTES),
+                             0 /* timeout */,
                              TALER_EC_MERCHANT_EXCHANGE_TRANSFERS_AWAITING_LIST,
-                             false,
-                             false);
+                             "Initiated GET with exchange",
+                             true);
 }
 
 
@@ -970,7 +967,6 @@ exchange_request (void *cls)
  * @param exchange_url base URL of the exchange that initiated the transfer
  * @param payto_uri account of the merchant that received the transfer
  * @param wtid wire transfer subject identifying the aggregation
- * @param total total amount that was wired
  * @param next_attempt when should we next try to interact with the exchange
  */
 static void
@@ -981,7 +977,6 @@ start_inquiry (
   const char *exchange_url,
   struct TALER_FullPayto payto_uri,
   const struct TALER_WireTransferIdentifierRawP *wtid,
-  const struct TALER_Amount *total,
   struct GNUNET_TIME_Absolute next_attempt)
 {
   struct Exchange *e;
@@ -996,6 +991,8 @@ start_inquiry (
                                       NULL);
     return;
   }
+  active_inquiries++;
+
   e = find_exchange (exchange_url);
   for (w = e->w_head; NULL != w; w = w->next)
   {
@@ -1009,13 +1006,11 @@ start_inquiry (
     }
   }
 
-  active_inquiries++;
   w = GNUNET_new (struct Inquiry);
   w->payto_uri.full_payto = GNUNET_strdup (payto_uri.full_payto);
   w->instance_id = GNUNET_strdup (instance_id);
   w->rowid = rowid;
   w->wtid = *wtid;
-  w->total = *total;
   GNUNET_CONTAINER_DLL_insert (e->w_head,
                                e->w_tail,
                                w);
@@ -1027,9 +1022,10 @@ start_inquiry (
   update_transaction_status (w,
                              GNUNET_TIME_relative_to_absolute (
                                GNUNET_TIME_UNIT_MINUTES),
+                             0 /* timeout */,
                              TALER_EC_MERCHANT_EXCHANGE_TRANSFERS_AWAITING_KEYS,
-                             false,
-                             false);
+                             exchange_url,
+                             true);
 }
 
 
@@ -1201,7 +1197,7 @@ run (void *cls,
   {
     struct GNUNET_DB_EventHeaderP es = {
       .size = htons (sizeof (es)),
-      .type = htons (TALER_DBEVENT_MERCHANT_WIRE_TRANSFER_CONFIRMED)
+      .type = htons (TALER_DBEVENT_MERCHANT_WIRE_TRANSFER_EXPECTED)
     };
 
     eh = db_plugin->event_listen (db_plugin->cls,
