@@ -1,6 +1,6 @@
 --
 -- This file is part of TALER
--- Copyright (C) 2022-2023 Taler Systems SA
+-- Copyright (C) 2024 Taler Systems SA
 --
 -- TALER is free software; you can redistribute it and/or modify it under the
 -- terms of the GNU General Public License as published by the Free Software
@@ -16,9 +16,15 @@
 
 BEGIN;
 -- Check patch versioning is in place.
--- SELECT _v.register_patch('merchant-0004', NULL, NULL);
+SELECT _v.register_patch('merchant-0004', NULL, NULL);
 
 SET search_path TO merchant;
 
+DROP TABLE merchant_reward_pickup_signatures;
+DROP TABLE merchant_reward_pickups;
+DROP TABLE merchant_rewards;
+DROP TABLE merchant_reward_reserve_keys;
+DROP TABLE merchant_reward_reserves;
+
 
 COMMIT;
