@@ -30,6 +30,7 @@ CREATE FUNCTION merchant_do_account_kyc_set_status (
   IN in_kyc_ok BOOL,
   IN in_notify_str TEXT,
   IN in_notify2_str TEXT,
+  IN in_rule_gen INT8,
   OUT out_no_instance BOOL,
   OUT out_no_account BOOL)
 LANGUAGE plpgsql
@@ -74,6 +75,7 @@ UPDATE merchant_kyc
       ,exchange_http_status=in_exchange_http_status
       ,exchange_ec_code=in_exchange_ec_code
       ,access_token=in_access_token
+      ,last_rule_gen=in_rule_gen
  WHERE account_serial=my_account_serial
    AND exchange_url=in_exchange_url;
 
@@ -89,7 +91,8 @@ THEN
     ,aml_review
     ,exchange_http_status
     ,exchange_ec_code
-    ,access_token)
+    ,access_token
+    ,last_rule_gen)
   VALUES
     (in_timestamp
     ,in_kyc_ok
@@ -99,7 +102,8 @@ THEN
     ,in_aml_active
     ,in_exchange_http_status
     ,in_exchange_ec_code
-    ,in_access_token);
+    ,in_access_token
+    ,in_rule_gen);
 END IF;
 
 EXECUTE FORMAT (
