@@ -43,11 +43,11 @@ TMH_PG_delete_otp (void *cls,
            "delete_otp",
            "DELETE"
            " FROM merchant_otp_devices"
-           " WHERE merchant_otp_devices.merchant_serial="
+           " WHERE merchant_serial="
            "     (SELECT merchant_serial "
            "        FROM merchant_instances"
            "        WHERE merchant_id=$1)"
-           "   AND merchant_otp_devices.otp_id=$2");
+           "   AND otp_id=$2");
   return GNUNET_PQ_eval_prepared_non_select (pg->conn,
                                              "delete_otp",
                                              params);
