@@ -27,20 +27,28 @@
 
 
 /**
+ * Insert new key pair for a token family.
+ *
  * @param cls closure
+ * @param merchant_id instance name
  * @param token_family_slug slug of the token family to insert the key for
  * @param pub public key to insert
  * @param priv private key to insert
- * @param valid_after start of validity period for this key
- * @param valid_before end of validity period for this key
+ * @param key_expires when does the private key expire (because
+ *     the validity period of the next token family key starts)
+ * @param valid_after start of validity period for signatures with this key
+ * @param valid_before end of validity period for signatures with this key
  * @return database result code
  */
 enum GNUNET_DB_QueryStatus
-TMH_PG_insert_token_family_key (void *cls,
-                                const char *token_family_slug,
-                                const struct TALER_TokenIssuePublicKeyP *pub,
-                                const struct TALER_TokenIssuePrivateKeyP *priv,
-                                const struct GNUNET_TIME_Timestamp valid_after,
-                                const struct GNUNET_TIME_Timestamp valid_before);
+TMH_PG_insert_token_family_key (
+  void *cls,
+  const char *merchant_id,
+  const char *token_family_slug,
+  const struct TALER_TokenIssuePublicKey *pub,
+  const struct TALER_TokenIssuePrivateKey *priv,
+  struct GNUNET_TIME_Timestamp key_expires,
+  struct GNUNET_TIME_Timestamp valid_after,
+  struct GNUNET_TIME_Timestamp valid_before);
 
 #endif
