@@ -1,6 +1,6 @@
 /*
    This file is part of TALER
-   Copyright (C) 2024 Taler Systems SA
+   Copyright (C) 2024, 2025 Taler Systems SA
 
    TALER is free software; you can redistribute it and/or modify it under the
    terms of the GNU General Public License as published by the Free Software
@@ -22,7 +22,7 @@
 #include <taler/taler_error_codes.h>
 #include <taler/taler_dbevents.h>
 #include <taler/taler_pq_lib.h>
-#include "pg_lookup_products.h"
+#include "pg_lookup_all_products.h"
 #include "pg_helper.h"
 
 /**
@@ -80,6 +80,8 @@ lookup_products_cb (void *cls,
                                     &product_id),
       GNUNET_PQ_result_spec_uint64 ("product_serial",
                                     &product_serial),
+      GNUNET_PQ_result_spec_string ("product_name",
+                                    &pd.product_name),
       GNUNET_PQ_result_spec_string ("description",
                                     &pd.description),
       TALER_PQ_result_spec_json ("description_i18n",
@@ -157,6 +159,7 @@ TMH_PG_lookup_all_products (void *cls,
            "SELECT"
            " description"
            ",description_i18n"
+           ",product_name"
            ",unit"
            ",price"
            ",taxes"
