@@ -64,8 +64,8 @@ struct LookupSpentTokensByOrderContext
  */
 static void
 lookup_spent_tokens_by_order_cb (void *cls,
-                             PGresult *result,
-                             unsigned int num_results)
+                                 PGresult *result,
+                                 unsigned int num_results)
 {
   struct LookupSpentTokensByOrderContext *ctx = cls;
 
@@ -88,8 +88,8 @@ lookup_spent_tokens_by_order_cb (void *cls,
                                             &use_pub),
       GNUNET_PQ_result_spec_auto_from_type ("token_sig",
                                             &use_sig),
-      // GNUNET_PQ_result_spec_unblinded_sig ("blind_sig",
-      //                                      &issue_sig),
+      GNUNET_PQ_result_spec_unblinded_sig ("blind_sig",
+                                           &issue_sig.signature),
       GNUNET_PQ_result_spec_end
     };
 
