@@ -56,7 +56,6 @@ TMH_PG_lookup_template (void *cls,
            " mt.template_description"
            ",mod.otp_id"
            ",mt.template_contract"
-           ",mt.required_currency"
            ",mt.editable_defaults"
            " FROM merchant_template mt"
            " JOIN merchant_instances mi"
@@ -85,10 +84,6 @@ TMH_PG_lookup_template (void *cls,
         GNUNET_PQ_result_spec_string ("otp_id",
                                       &td->otp_id),
         NULL),
-      GNUNET_PQ_result_spec_allow_null (
-        GNUNET_PQ_result_spec_string ("required_currency",
-                                      &td->required_currency),
-        NULL),
       TALER_PQ_result_spec_json ("template_contract",
                                  &td->template_contract),
       GNUNET_PQ_result_spec_allow_null (
