@@ -40,7 +40,7 @@
  * @param limit number of entries to return, negative for descending in execution time,
  *                positive for ascending in execution time
  * @param offset transfer_serial number of the transfer we want to offset from
- * @param verified filter transfers by verification status
+ * @param expected filter for transfers that were expected
  * @param cb function to call with detailed transfer data
  * @param cb_cls closure for @a cb
  * @return transaction status
@@ -53,7 +53,7 @@ TMH_PG_lookup_transfers (void *cls,
                          struct GNUNET_TIME_Timestamp after,
                          int64_t limit,
                          uint64_t offset,
-                         enum TALER_EXCHANGE_YesNoAll verified,
+                         enum TALER_EXCHANGE_YesNoAll expected,
                          TALER_MERCHANTDB_TransferCallback cb,
                          void *cb_cls);
 
