@@ -29,6 +29,7 @@
 enum GNUNET_DB_QueryStatus
 TMH_PG_select_exchange_keys (void *cls,
                              const char *exchange_url,
+                             struct GNUNET_TIME_Absolute *first_retry,
                              struct TALER_EXCHANGE_Keys **keys)
 {
   struct PostgresClosure *pg = cls;
@@ -38,6 +39,8 @@ TMH_PG_select_exchange_keys (void *cls,
   };
   json_t *jkeys;
   struct GNUNET_PQ_ResultSpec rs[] = {
+    GNUNET_PQ_result_spec_absolute_time ("first_retry",
+                                         first_retry),
     TALER_PQ_result_spec_json ("keys_json",
                                &jkeys),
     GNUNET_PQ_result_spec_end
@@ -48,7 +51,8 @@ TMH_PG_select_exchange_keys (void *cls,
   PREPARE (pg,
            "select_exchange_keys",
            "SELECT"
-           " keys_json"
+           " first_retry"
+           ",keys_json"
            " FROM merchant_exchange_keys"
            " WHERE exchange_url=$1;");
   qs = GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
