@@ -42,6 +42,8 @@ TMH_PG_update_instance (void *cls,
       &is->default_wire_transfer_delay),
     GNUNET_PQ_query_param_relative_time (
       &is->default_pay_delay),
+    GNUNET_PQ_query_param_relative_time (
+      &is->default_refund_delay),
     (NULL == is->website)
     ? GNUNET_PQ_query_param_null ()
     : GNUNET_PQ_query_param_string (is->website),
@@ -51,6 +53,11 @@ TMH_PG_update_instance (void *cls,
     (NULL == is->logo)
     ? GNUNET_PQ_query_param_null ()
     : GNUNET_PQ_query_param_string (is->logo),
+    (NULL == is->phone)
+    ? GNUNET_PQ_query_param_null ()
+    : GNUNET_PQ_query_param_string (is->phone),
+    GNUNET_PQ_query_param_bool (is->phone_validated),
+    GNUNET_PQ_query_param_bool (is->email_validated),
     GNUNET_PQ_query_param_end
   };
 
@@ -64,9 +71,13 @@ TMH_PG_update_instance (void *cls,
            ",use_stefan=$5"
            ",default_wire_transfer_delay=$6"
            ",default_pay_delay=$7"
-           ",website=$8"
-           ",email=$9"
-           ",logo=$10"
+           ",default_refund_delay=$8"
+           ",website=$9"
+           ",email=$10"
+           ",logo=$11"
+           ",phone_number=$12"
+           ",phone_validated=$13"
+           ",email_validated=$14"
            " WHERE merchant_id=$1");
   return GNUNET_PQ_eval_prepared_non_select (pg->conn,
                                              "update_instance",
