@@ -41,7 +41,9 @@ TMH_PG_update_token_family (
     GNUNET_PQ_query_param_string (details->name),
     GNUNET_PQ_query_param_string (details->description),
     TALER_PQ_query_param_json (details->description_i18n),
-    TALER_PQ_query_param_json (details->extra_data),
+    (details->extra_data == NULL)
+    ? GNUNET_PQ_query_param_null ()
+    : TALER_PQ_query_param_json (details->extra_data),
     GNUNET_PQ_query_param_timestamp (&details->valid_after),
     GNUNET_PQ_query_param_timestamp (&details->valid_before),
     GNUNET_PQ_query_param_end
