@@ -1,30 +1,28 @@
 BEGIN;
 SET search_path TO merchant;
-CREATE OR REPLACE FUNCTION merchant_insert_deposit_to_transfer (
+DROP FUNCTION IF EXISTS merchant_insert_deposit_to_transfer;
+CREATE FUNCTION merchant_insert_deposit_to_transfer (
   IN in_deposit_serial INT8,
-  IN in_amount_with_fee taler_amount_currency,
+  IN in_coin_contribution taler_amount_currency,
   IN in_execution_time INT8,
+  IN in_exchange_url TEXT,
+  IN in_h_wire BYTEA,
   IN in_exchange_sig BYTEA,
   IN in_exchange_pub BYTEA,
   IN in_wtid BYTEA,
-  OUT out_wire_pending_cleared BOOL,
-  OUT out_conflict BOOL,
-  OUT out_no_exchange_pub BOOL)
+  OUT out_dummy BOOL)
 LANGUAGE plpgsql
 AS $$
 DECLARE
   my_signkey_serial INT8;
-DECLARE
-  my_confirmed BOOL;
-DECLARE
+  my_account_serial INT8;
   my_decose INT8;
-DECLARE
   my_order_serial INT8;
-DECLARE
-  my_merchant_serial INT8;
-DECLARE
+  my_expected_credit_serial INT8;
+  my_wire_pending_cleared BOOL;
   my_order_id TEXT;
 BEGIN
+  out_dummy=FALSE;
 SELECT signkey_serial
   INTO my_signkey_serial
   FROM merchant_exchange_signing_keys
@@ -33,92 +31,77 @@ SELECT signkey_serial
    LIMIT 1;
 IF NOT FOUND
 THEN
-  out_no_exchange_pub=TRUE;
-  out_conflict=FALSE;
-  out_wire_pending_cleared=FALSE;
+  UPDATE merchant_deposits
+     SET settlement_last_ec=2029 
+        ,settlement_last_http_status=200
+        ,settlement_last_detail=ENCODE(in_exchange_pub, 'hex')
+        ,settlement_wtid=in_wtid
+        ,settlement_retry_needed=TRUE
+        ,settlement_retry_time=(EXTRACT(epoch FROM (CURRENT_TIME + interval '8 hours')) * 1000000)::INT8
+   WHERE deposit_serial=in_deposit_serial;
   RETURN;
 END IF;
-out_no_exchange_pub=FALSE;
-INSERT INTO merchant_deposit_to_transfer
-  (deposit_serial
-  ,coin_contribution_value
-  ,wtid
-  ,execution_time
-  ,signkey_serial
-  ,exchange_sig
-  )
-  VALUES
-  (in_deposit_serial
-  ,in_amount_with_fee
-  ,in_wtid
-  ,in_execution_time
-  ,my_signkey_serial
-  ,in_exchange_sig
-  )
-  ON CONFLICT DO NOTHING;
-IF NOT FOUND
-THEN
-  PERFORM FROM merchant_deposit_to_transfer
-    WHERE deposit_serial=in_deposit_serial
-      AND wtid=in_wtid
-      AND signkey_serial=my_signkey_serial
-      AND exchange_sig=in_exchange_sig;
-END IF;
-IF NOT FOUND
-THEN
-  out_conflict=TRUE;
-  out_wire_pending_cleared=FALSE;
-  RETURN;
-END IF;
-out_conflict=FALSE;
-PERFORM
-  FROM merchant_transfers mt
-  JOIN merchant_transfer_to_coin mtc
-    USING (credit_serial)
-  WHERE mt.wtid=in_wtid
-    AND mt.confirmed
-    AND mtc.deposit_serial=in_deposit_serial;
-IF NOT FOUND
-THEN
-  out_wire_pending_cleared=FALSE;
-  RETURN;
-END IF;
-RAISE NOTICE 'checking affected deposit confirmation for completion';
 SELECT deposit_confirmation_serial
   INTO my_decose
   FROM merchant_deposits
  WHERE deposit_serial=in_deposit_serial;
-UPDATE merchant_deposit_confirmations
-  SET wire_pending=FALSE
-  WHERE (deposit_confirmation_serial=my_decose)
-    AND NOT EXISTS
-    (SELECT 1
-      FROM merchant_deposits md
-      LEFT JOIN merchant_deposit_to_transfer mdtt
-        USING (deposit_serial)
-      WHERE md.deposit_confirmation_serial=my_decose
-        AND mdtt.signkey_serial IS NULL);
+SELECT account_serial
+  INTO my_account_serial
+  FROM merchant_deposit_confirmations mdc
+  JOIN merchant_accounts ma
+    USING (account_serial)
+ WHERE mdc.deposit_confirmation_serial=my_decose
+   AND ma.h_wire=in_h_wire;
 IF NOT FOUND
 THEN
-  out_wire_pending_cleared=FALSE;
+  UPDATE merchant_deposits
+     SET settlement_last_ec=2558 
+        ,settlement_last_http_status=200
+        ,settlement_last_detail=ENCODE(in_h_wire, 'hex')
+        ,settlement_wtid=in_wtid
+        ,settlement_retry_needed=FALSE
+        ,settlement_coin_contribution=in_coin_contribution
+        ,signkey_serial=my_signkey_serial
+        ,settlement_exchange_sig=in_exchange_sig
+   WHERE deposit_serial=in_deposit_serial;
   RETURN;
 END IF;
-out_wire_pending_cleared=TRUE;
-RAISE NOTICE 'checking affected contracts for completion';
-SELECT deposit_confirmation_serial
-  INTO my_order_serial
-  FROM merchant_deposit_confirmations
- WHERE deposit_confirmation_serial=my_decose;
-UPDATE merchant_contract_terms
-  SET wired=TRUE
-  WHERE (order_serial=my_order_serial)
-    AND NOT EXISTS
-    (SELECT 1
-       FROM merchant_deposit_confirmations mdc
-      WHERE mdc.wire_pending
-        AND mdc.order_serial=my_order_serial);
+SELECT expected_credit_serial
+  INTO my_expected_credit_serial
+  FROM merchant_expected_transfers
+  WHERE wtid=in_wtid
+    AND exchange_url=in_exchange_url
+    AND account_serial=my_account_serial;
+IF NOT FOUND
+THEN
+  INSERT INTO merchant_expected_transfers
+    (exchange_url
+    ,wtid
+    ,account_serial
+    ,expected_time)
+   VALUES
+    (in_exchange_url
+    ,in_wtid
+    ,my_account_serial
+    ,in_execution_time)
+   RETURNING expected_credit_serial
+     INTO my_expected_credit_serial;
+END IF;
+UPDATE merchant_deposits
+   SET settlement_last_ec=0
+      ,settlement_last_http_status=200
+      ,settlement_last_detail=NULL
+      ,settlement_wtid=in_wtid
+      ,settlement_retry_needed=FALSE
+      ,settlement_coin_contribution=in_coin_contribution
+      ,settlement_expected_credit_serial=my_expected_credit_serial
+      ,signkey_serial=my_signkey_serial
+      ,settlement_exchange_sig=in_exchange_sig
+ WHERE deposit_serial=in_deposit_serial;
+NOTIFY XR6849FMRD2AJFY1E2YY0GWA8GN0YT407Z66WHJB0SAKJWF8G2Q60;
 END $$;
-CREATE OR REPLACE FUNCTION merchant_do_insert_product (
+DROP FUNCTION IF EXISTS merchant_do_insert_product;
+CREATE FUNCTION merchant_do_insert_product (
   IN in_instance_id TEXT,
   IN in_product_id TEXT,
   IN in_description TEXT,
@@ -132,6 +115,7 @@ CREATE OR REPLACE FUNCTION merchant_do_insert_product (
   IN in_next_restock INT8,
   IN in_minimum_age INT4,
   IN ina_categories INT8[],
+  IN in_product_name TEXT,
   OUT out_no_instance BOOL,
   OUT out_conflict BOOL,
   OUT out_no_cat INT8)
@@ -158,6 +142,7 @@ out_no_instance=FALSE;
 INSERT INTO merchant_inventory
  (merchant_serial
  ,product_id
+ ,product_name
  ,description
  ,description_i18n
  ,unit
@@ -171,6 +156,7 @@ INSERT INTO merchant_inventory
 ) VALUES (
   my_merchant_id
  ,in_product_id
+ ,in_product_name
  ,in_description
  ,in_description_i18n
  ,in_unit
@@ -191,6 +177,7 @@ THEN
     FROM merchant_inventory
   WHERE merchant_serial=my_merchant_id
     AND product_id=in_product_id
+    AND product_name=in_product_name
     AND description=in_description
     AND description_i18n=in_description_i18n
     AND unit=in_unit
@@ -254,7 +241,114 @@ LOOP
 END LOOP;
 out_no_cat=NULL;
 END $$;
-CREATE OR REPLACE FUNCTION merchant_do_insert_transfer_details (
+DROP FUNCTION IF EXISTS merchant_do_insert_issued_token;
+CREATE FUNCTION merchant_do_insert_issued_token (
+  IN in_h_issue_pub BYTEA,
+  IN in_h_contract_terms BYTEA,
+  IN in_blind_sig BYTEA,
+  OUT out_no_family BOOL,
+  OUT out_existed BOOL)
+LANGUAGE plpgsql
+AS $$
+DECLARE
+  my_rec RECORD;
+  my_tfk_serial INT8;
+  my_tf_serial INT8;
+BEGIN
+SELECT token_family_key_serial
+      ,token_family_serial
+  INTO my_rec
+  FROM merchant_token_family_keys
+ WHERE h_pub = in_h_issue_pub;
+IF NOT FOUND
+THEN
+  out_no_family = TRUE;
+  out_existed = FALSE;
+  return;
+END IF;
+my_tfk_serial = my_rec.token_family_key_serial;
+my_tf_serial = my_rec.token_family_serial;
+out_no_family = FALSE;
+INSERT INTO merchant_issued_tokens
+  (token_family_key_serial
+  ,h_contract_terms
+  ,blind_sig
+  ) VALUES
+  (my_tfk_serial
+  ,in_h_contract_terms
+  ,in_blind_sig)
+  ON CONFLICT DO NOTHING;
+IF NOT FOUND
+THEN
+  out_existed = TRUE;
+  return;
+END IF;
+out_existed = FALSE;
+UPDATE merchant_token_families
+   SET issued=issued+1
+  WHERE token_family_serial=my_tf_serial;
+END $$;
+DROP FUNCTION IF EXISTS merchant_do_insert_spent_token;
+CREATE FUNCTION merchant_do_insert_spent_token (
+  IN in_h_contract_terms BYTEA,
+  IN in_h_issue_pub BYTEA,
+  IN in_use_pub BYTEA,
+  IN in_use_sig BYTEA,
+  IN in_issue_sig BYTEA,
+  OUT out_no_family BOOL,
+  OUT out_conflict BOOL)
+LANGUAGE plpgsql
+AS $$
+DECLARE
+  my_rec RECORD;
+  my_tfk_serial INT8;
+  my_tf_serial INT8;
+BEGIN
+SELECT token_family_key_serial
+      ,token_family_serial
+  INTO my_rec
+  FROM merchant_token_family_keys
+ WHERE h_pub = in_h_issue_pub;
+IF NOT FOUND
+THEN
+  out_no_family = TRUE;
+  out_conflict = FALSE;
+  return;
+END IF;
+out_no_family = FALSE;
+my_tfk_serial = my_rec.token_family_key_serial;
+my_tf_serial = my_rec.token_family_serial;
+INSERT INTO merchant_used_tokens
+  (token_family_key_serial
+  ,h_contract_terms
+  ,token_pub
+  ,token_sig
+  ,blind_sig
+  ) VALUES
+  (my_tfk_serial
+  ,in_h_contract_terms
+  ,in_use_pub
+  ,in_use_sig
+  ,in_issue_sig)
+  ON CONFLICT DO NOTHING;
+IF NOT FOUND
+THEN
+  PERFORM FROM merchant_used_tokens
+    WHERE token_family_key_serial=my_tfk_serial
+      AND h_contract_terms=in_h_contract_terms
+      AND token_pub=in_use_pub
+      AND token_sig=in_use_sig
+      AND blind_sig=in_issue_sig;
+  out_conflict = NOT FOUND;
+  return;
+END IF;
+out_conflict = FALSE;
+UPDATE merchant_token_families
+   SET used=used+1
+ WHERE token_family_serial=my_tf_serial;
+END $$;
+DROP FUNCTION IF EXISTS merchant_do_insert_transfer_details;
+CREATE FUNCTION merchant_do_insert_transfer_details (
   IN in_instance_id TEXT,
   IN in_exchange_url TEXT,
   IN in_payto_uri TEXT,
@@ -278,9 +372,10 @@ AS $$
 DECLARE
   my_merchant_id INT8;
   my_signkey_serial INT8;
-  my_credit_serial INT8;
+  my_expected_credit_serial INT8;
   my_affected_orders RECORD;
   my_merchant_serial INT8;
+  my_decose INT8;
   my_order_id TEXT;
   i INT8;
   curs CURSOR (arg_coin_pub BYTEA) FOR
@@ -309,9 +404,9 @@ THEN
   RETURN;
 END IF;
 out_no_instance=FALSE;
-SELECT credit_serial
-  INTO my_credit_serial
-  FROM merchant_transfers
+SELECT expected_credit_serial
+  INTO my_expected_credit_serial
+  FROM merchant_expected_transfers
  WHERE exchange_url=in_exchange_url
      AND wtid=in_wtid
      AND account_serial=
@@ -344,14 +439,14 @@ THEN
 END IF;
 out_no_exchange=FALSE;
 INSERT INTO merchant_transfer_signatures
-  (credit_serial
+  (expected_credit_serial
   ,signkey_serial
   ,credit_amount
   ,wire_fee
   ,execution_time
   ,exchange_sig)
   VALUES
-   (my_credit_serial
+   (my_expected_credit_serial
    ,my_signkey_serial
    ,in_total_amount
    ,in_wire_fee
@@ -362,7 +457,7 @@ IF NOT FOUND
 THEN
   PERFORM 1
     FROM merchant_transfer_signatures
-    WHERE credit_serial=my_credit_serial
+    WHERE expected_credit_serial=my_expected_credit_serial
       AND signkey_serial=my_signkey_serial
       AND credit_amount=in_total_amount
       AND wire_fee=in_wire_fee
@@ -386,15 +481,15 @@ LOOP
   ini_deposit_fee=ina_deposit_fees[i];
   ini_coin_pub=ina_coin_pubs[i];
   ini_contract_term=ina_contract_terms[i];
-  INSERT INTO merchant_transfer_to_coin
+  INSERT INTO merchant_expected_transfer_to_coin
     (deposit_serial
-    ,credit_serial
+    ,expected_credit_serial
     ,offset_in_exchange_list
     ,exchange_deposit_value
     ,exchange_deposit_fee)
     SELECT
         dep.deposit_serial
-       ,my_credit_serial
+       ,my_expected_credit_serial
        ,i
        ,ini_coin_value
        ,ini_deposit_fee
@@ -412,50 +507,57 @@ LOOP
     FETCH NEXT FROM curs INTO my_affected_orders;
     EXIT WHEN NOT FOUND;
     RAISE NOTICE 'checking affected order for completion';
-    UPDATE merchant_deposit_confirmations
-       SET wire_pending=FALSE
-     WHERE (deposit_confirmation_serial=my_affected_orders.deposit_confirmation_serial)
-       AND NOT EXISTS
-       (SELECT 1
-          FROM merchant_deposits md
-          LEFT JOIN merchant_deposit_to_transfer mdtt
-            USING (deposit_serial)
-          WHERE md.deposit_confirmation_serial=my_affected_orders.deposit_confirmation_serial
-            AND mdtt.wtid IS NULL);
-    IF FOUND
+    my_decose=my_affected_orders.deposit_confirmation_serial;
+    PERFORM FROM merchant_deposits md
+       WHERE md.deposit_confirmation_serial=my_decose
+         AND settlement_retry_needed
+          OR settlement_wtid IS NULL;
+    IF NOT FOUND
     THEN
-      UPDATE merchant_contract_terms
-         SET wired=TRUE
-       WHERE (order_serial=my_affected_orders.order_serial)
-         AND NOT EXISTS
-         (SELECT 1
-            FROM merchant_deposit_confirmations mdc
-             WHERE mdc.wire_pending
-               AND mdc.order_serial=my_affected_orders.order_serial);
-      SELECT merchant_serial, order_id
-        INTO my_merchant_serial, my_order_id
-        FROM merchant_contract_terms
-       WHERE order_serial=my_affected_orders.order_serial;
-      INSERT INTO merchant_pending_webhooks
-      (merchant_serial
-      ,webhook_serial
-      ,url
-      ,http_method
-      ,body)
-      SELECT mw.merchant_serial
-            ,mw.webhook_serial
-            ,mw.url
-            ,mw.http_method
-            ,replace_placeholder(mw.body_template, 'order_id', my_order_id)::TEXT
-      FROM merchant_webhook mw
-      WHERE mw.event_type = 'order_settled'
-        AND mw.merchant_serial = my_merchant_serial;
-    END IF;
+      UPDATE merchant_deposit_confirmations
+         SET wire_pending=FALSE
+       WHERE (deposit_confirmation_serial=my_decose);
+      IF FOUND
+      THEN
+        RAISE NOTICE 'checking affected contract for completion';
+        PERFORM FROM merchant_deposit_confirmations mdc
+               WHERE mdc.wire_pending
+                 AND mdc.order_serial=my_affected_orders.order_serial;
+        IF NOT FOUND
+        THEN
+          UPDATE merchant_contract_terms
+             SET wired=TRUE
+           WHERE (order_serial=my_affected_orders.order_serial);
+          SELECT merchant_serial, order_id
+            INTO my_merchant_serial, my_order_id
+            FROM merchant_contract_terms
+           WHERE order_serial=my_affected_orders.order_serial;
+          INSERT INTO merchant_pending_webhooks
+           (merchant_serial
+           ,webhook_serial
+           ,url
+           ,http_method
+           ,body)
+           SELECT mw.merchant_serial
+                 ,mw.webhook_serial
+                 ,mw.url
+                 ,mw.http_method
+                 ,replace_placeholder(
+                  replace_placeholder(mw.body_template, 'order_id', my_order_id),
+                  'wtid', encode(in_wtid, 'hex')
+                  )::TEXT
+             FROM merchant_webhook mw
+            WHERE mw.event_type = 'order_settled'
+              AND mw.merchant_serial = my_merchant_serial;
+        END IF; 
+      END IF; 
+    END IF; 
   END LOOP; 
   CLOSE curs;
 END LOOP; 
 END $$;
-CREATE OR REPLACE FUNCTION merchant_do_update_product (
+DROP FUNCTION IF EXISTS merchant_do_update_product;
+CREATE FUNCTION merchant_do_update_product (
   IN in_instance_id TEXT,
   IN in_product_id TEXT,
   IN in_description TEXT,
@@ -470,6 +572,7 @@ CREATE OR REPLACE FUNCTION merchant_do_update_product (
   IN in_next_restock INT8,
   IN in_minimum_age INT4,
   IN ina_categories INT8[],
+  IN in_product_name TEXT,
   OUT out_no_instance BOOL,
   OUT out_no_product BOOL,
   OUT out_lost_reduced BOOL,
@@ -544,6 +647,7 @@ END LOOP;
 UPDATE merchant_inventory SET
    description=in_description
   ,description_i18n=in_description_i18n
+  ,product_name=in_product_name
   ,unit=in_unit
   ,image=in_image
   ,taxes=in_taxes
@@ -557,6 +661,64 @@ UPDATE merchant_inventory SET
    AND product_serial=my_product_serial; 
 ASSERT FOUND,'SELECTED it earlier, should UPDATE it now';
 END $$;
+DROP FUNCTION IF EXISTS merchant_do_solve_mfa_challenge;
+CREATE FUNCTION merchant_do_solve_mfa_challenge (
+  IN in_challenge_id INT8,
+  IN in_h_body BYTEA,
+  IN in_solution TEXT,
+  IN in_now INT8,
+  OUT out_solved BOOLEAN,
+  OUT out_retry_counter INT4
+)
+LANGUAGE plpgsql
+AS $$
+DECLARE
+  my_confirmation_date INT8;
+DECLARE
+  my_rec RECORD;
+BEGIN
+  SELECT
+    tc.confirmation_date
+   ,tc.retry_counter
+   ,(tc.code = in_solution) AS solved
+  INTO
+    my_rec
+  FROM tan_challenges tc
+  WHERE tc.challenge_id = in_challenge_id
+    AND tc.h_body = in_h_body
+    AND tc.expiration_date > in_now;
+  IF NOT FOUND
+  THEN
+    out_solved = FALSE;
+    RETURN;
+  END IF;
+  my_confirmation_date = my_rec.confirmation_date;
+  out_retry_counter = my_rec.retry_counter;
+  out_solved = my_rec.solved;
+  IF my_confirmation_date IS NOT NULL
+  THEN
+    out_solved = TRUE;
+    RETURN;
+  END IF;
+  IF (0 = out_retry_counter)
+  THEN
+    out_solved = FALSE;
+    RETURN;
+  END IF;
+  IF out_solved
+  THEN
+    my_confirmation_date = in_now;
+    UPDATE tan_challenges
+       SET confirmation_date = my_confirmation_date
+     WHERE challenge_id = in_challenge_id;
+  ELSE
+    out_retry_counter = out_retry_counter - 1;
+    UPDATE tan_challenges
+       SET retry_counter = out_retry_counter
+     WHERE challenge_id = in_challenge_id;
+  END IF;
+END;
+$$;
 DROP FUNCTION IF EXISTS merchant_do_account_kyc_set_status;
 CREATE FUNCTION merchant_do_account_kyc_set_status (
   IN in_merchant_id TEXT,
@@ -1108,7 +1270,7 @@ END $$;
 COMMENT ON PROCEDURE merchant_do_bump_amount_stat
   IS 'Updates an amount statistic (bucket or interval)';
 DROP FUNCTION IF EXISTS merchant_statistic_interval_number_get;
-CREATE OR REPLACE FUNCTION merchant_statistic_interval_number_get (
+CREATE FUNCTION merchant_statistic_interval_number_get (
   IN in_slug TEXT,
   IN in_instance_id TEXT
 )
@@ -1236,7 +1398,7 @@ END $$;
 COMMENT ON FUNCTION merchant_statistic_interval_number_get
   IS 'Returns deposit statistic tracking deposited amounts over certain time intervals; we first trim the stored data to only track what is still in-range, and then return the remaining value for each range';
 DROP FUNCTION IF EXISTS merchant_statistic_interval_amount_get;
-CREATE OR REPLACE FUNCTION merchant_statistic_interval_amount_get (
+CREATE FUNCTION merchant_statistic_interval_amount_get (
   IN in_slug TEXT,
   IN in_instance_id TEXT
 )
@@ -1661,14 +1823,37 @@ BEGIN
 END $$;
 COMMENT ON PROCEDURE merchant_statistic_bucket_gc
   IS 'Performs garbage collection of the merchant_statistic_bucket_counter and merchant_statistic_bucket_amount tables';
-DROP PROCEDURE IF EXISTS merchant_do_gc;
-CREATE PROCEDURE merchant_do_gc()
+DROP FUNCTION IF EXISTS merchant_statistics_bucket_end;
+CREATE FUNCTION merchant_statistics_bucket_end (
+  IN in_bucket_start INT8,
+  IN in_range statistic_range,
+  OUT out_bucket_end INT8
+)
 LANGUAGE plpgsql
 AS $$
 BEGIN
+    IF in_range='quarter'
+    THEN
+      out_bucket_end = EXTRACT(EPOCH FROM CAST(date_trunc('quarter', to_timestamp(in_bucket_start)::date) + interval '3 months' AS date));
+    ELSE
+      out_bucket_end = EXTRACT(EPOCH FROM CAST(to_timestamp(in_bucket_start)::date + ('1 ' || in_range)::interval AS date));
+    END IF;
+END $$;
+COMMENT ON FUNCTION merchant_statistics_bucket_end
+IS 'computes the end time of the bucket for an event at the current time given the desired bucket range';
+DROP PROCEDURE IF EXISTS merchant_do_gc;
+CREATE PROCEDURE merchant_do_gc(in_now INT8)
+LANGUAGE plpgsql
+AS $$
+BEGIN
+  DELETE FROM merchant_instances
+    WHERE validation_needed
+      AND validation_expiration < in_now;
   CALL merchant_statistic_amount_gc ();
   CALL merchant_statistic_bucket_gc ();
   CALL merchant_statistic_counter_gc ();
+  DELETE FROM tan_challenges
+    WHERE expiration_date < in_now;
 END $$;
 COMMENT ON PROCEDURE merchant_do_gc
   IS 'calls all other garbage collection subroutines';
