@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2024 Taler Systems SA
+  Copyright (C) 2014--2025 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -93,6 +93,9 @@ handle_get_product_finished (void *cls,
   case MHD_HTTP_OK:
     {
       struct GNUNET_JSON_Specification spec[] = {
+        GNUNET_JSON_spec_string (
+          "product_name",
+          &pgr.details.ok.product_name),
         GNUNET_JSON_spec_string (
           "description",
           &pgr.details.ok.description),
