@@ -165,43 +165,27 @@ TALER_MERCHANT_instances_post (
   const char *backend_url,
   const char *instance_id,
   const char *name,
-  enum TALER_KYCLOGIC_KycUserType ut,
   const json_t *address,
   const json_t *jurisdiction,
   bool use_stefan,
   struct GNUNET_TIME_Relative default_wire_transfer_delay,
   struct GNUNET_TIME_Relative default_pay_delay,
-  const char *auth_token,
+  struct GNUNET_TIME_Relative default_refund_delay,
+  const char *auth_password,
   TALER_MERCHANT_InstancesPostCallback cb,
   void *cb_cls)
 {
   struct TALER_MERCHANT_InstancesPostHandle *iph;
   json_t *req_obj;
   json_t *auth_obj;
-  const char *uts;
 
-  uts = TALER_KYCLOGIC_kyc_user_type2s (ut);
-  if (NULL == uts)
+  if (NULL != auth_password)
   {
-    GNUNET_break (0);
-    return NULL;
-  }
-  if (NULL != auth_token)
-  {
-    if (0 != strncasecmp (RFC_8959_PREFIX,
-                          auth_token,
-                          strlen (RFC_8959_PREFIX)))
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Authentication token must start with `%s'\n",
-                  RFC_8959_PREFIX);
-      return NULL;
-    }
     auth_obj = GNUNET_JSON_PACK (
       GNUNET_JSON_pack_string ("method",
                                "token"),
-      GNUNET_JSON_pack_string ("token",
-                               auth_token));
+      GNUNET_JSON_pack_string ("password",
+                               auth_password));
   }
   else
   {
@@ -219,8 +203,6 @@ TALER_MERCHANT_instances_post (
                              instance_id),
     GNUNET_JSON_pack_string ("name",
                              name),
-    GNUNET_JSON_pack_string ("user_type",
-                             uts),
     GNUNET_JSON_pack_object_incref ("address",
                                     (json_t *) address),
     GNUNET_JSON_pack_object_incref ("jurisdiction",
@@ -231,6 +213,8 @@ TALER_MERCHANT_instances_post (
                                default_wire_transfer_delay),
     GNUNET_JSON_pack_time_rel ("default_pay_delay",
                                default_pay_delay),
+    GNUNET_JSON_pack_time_rel ("default_refund_delay",
+                               default_refund_delay),
     GNUNET_JSON_pack_object_steal ("auth",
                                    auth_obj));
   iph = GNUNET_new (struct TALER_MERCHANT_InstancesPostHandle);
