@@ -341,6 +341,7 @@ TALER_MERCHANT_order_abort (
     j_coin = GNUNET_JSON_PACK (
       GNUNET_JSON_pack_data_auto ("coin_pub",
                                   &ac->coin_pub),
+      /* FIXME: no longer needed since **v18**, remove eventually! */
       TALER_JSON_pack_amount ("contribution",
                               &ac->amount_with_fee),
       GNUNET_JSON_pack_string ("exchange_url",
@@ -417,7 +418,8 @@ TALER_MERCHANT_order_abort (
 
 
 void
-TALER_MERCHANT_order_abort_cancel (struct TALER_MERCHANT_OrderAbortHandle *oah)
+TALER_MERCHANT_order_abort_cancel (
+  struct TALER_MERCHANT_OrderAbortHandle *oah)
 {
   if (NULL != oah->job)
   {
