@@ -198,6 +198,10 @@ TALER_MERCHANT_products_post3 (
   req_obj = GNUNET_JSON_PACK (
     GNUNET_JSON_pack_string ("product_id",
                              product_id),
+    /* FIXME: once we move to the new-style API,
+       allow applications to set the product name properly! */
+    GNUNET_JSON_pack_string ("product_name",
+                             description),
     GNUNET_JSON_pack_string ("description",
                              description),
     GNUNET_JSON_pack_allow_null (
