@@ -129,6 +129,10 @@ handle_post_templates_finished (void *cls,
        happen, we should pass the JSON reply to the
        application */
     break;
+  case MHD_HTTP_CONFLICT:
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     hr.ec = TALER_JSON_get_error_code (json);
     hr.hint = TALER_JSON_get_error_hint (json);
@@ -199,8 +203,7 @@ TALER_MERCHANT_templates_post (
   const char *backend_url,
   const char *template_id,
   const char *template_description,
-  const char *pos_key,
-  enum TALER_MerchantConfirmationAlgorithm pos_algorithm,
+  const char *otp_id,
   const json_t *template_contract,
   TALER_MERCHANT_TemplatesPostCallback cb,
   void *cb_cls)
@@ -218,11 +221,9 @@ TALER_MERCHANT_templates_post (
                              template_id),
     GNUNET_JSON_pack_string ("template_description",
                              template_description),
-    GNUNET_JSON_pack_uint64 ("pos_algorithm",
-                             (uint32_t) pos_algorithm),
     GNUNET_JSON_pack_allow_null (
-      GNUNET_JSON_pack_string ("pos_key",
-                               pos_key)),
+      GNUNET_JSON_pack_string ("otp_id",
+                               otp_id)),
     GNUNET_JSON_pack_object_incref ("template_contract",
                                     (json_t *) template_contract));
   tph = GNUNET_new (struct TALER_MERCHANT_TemplatesPostHandle);
