@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2023 Taler Systems SA
+  Copyright (C) 2023, 2025 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -65,16 +65,6 @@ run (void *cls,
     global_ret = -1;
     return;
   }
-  if (0 != strncmp (pw,
-                    RFC_8959_PREFIX,
-                    strlen (RFC_8959_PREFIX)))
-  {
-    fprintf (stderr,
-             "Invalid password specified, does not begin with `%s'\n",
-             RFC_8959_PREFIX);
-    global_ret = 1;
-    return;
-  }
   if (NULL == instance)
     instance = GNUNET_strdup ("admin");
   cfg = GNUNET_CONFIGURATION_dup (config);
@@ -91,17 +81,9 @@ run (void *cls,
   GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_NONCE,
                               &ias.auth_salt,
                               sizeof (ias.auth_salt));
-  GNUNET_assert (GNUNET_YES ==
-                 GNUNET_CRYPTO_kdf (&ias.auth_hash,
-                                    sizeof (ias.auth_hash),
-                                    &ias.auth_salt,
-                                    sizeof (ias.auth_salt),
-                                    pw,
-                                    strlen (pw),
-                                    "merchant-instance-auth",
-                                    strlen ("merchant-instance-auth"),
-                                    NULL,
-                                    0));
+  TALER_merchant_instance_auth_hash_with_salt (&ias.auth_hash,
+                                               &ias.auth_salt,
+                                               pw);
   if (GNUNET_OK !=
       plugin->connect (plugin->cls))
   {
@@ -131,10 +113,84 @@ run (void *cls,
     }
     break;
   case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
-    fprintf (stderr,
-             "Instance `%s' unknown, cannot reset token\n",
-             instance);
-    global_ret = 2;
+    if (0 ==
+        strcmp (instance,
+                "admin"))
+    {
+      struct TALER_MerchantPrivateKeyP merchant_priv;
+      struct TALER_MerchantPublicKeyP merchant_pub;
+      struct TALER_MERCHANTDB_InstanceSettings is = {
+        .id = (char *) "admin",
+        .name = (char *) "Administrator",
+        .use_stefan = true,
+        .address = json_object (),
+        .jurisdiction = json_object (),
+      };
+
+      if (GNUNET_OK !=
+          GNUNET_CONFIGURATION_get_value_time (config,
+                                               "merchant",
+                                               "DEFAULT_WIRE_TRANSFER_DELAY",
+                                               &is.default_wire_transfer_delay)
+          )
+      {
+        is.default_wire_transfer_delay = GNUNET_TIME_UNIT_MONTHS;
+      }
+      if (GNUNET_OK !=
+          GNUNET_CONFIGURATION_get_value_time (config,
+                                               "merchant",
+                                               "DEFAULT_PAY_DELAY",
+                                               &is.default_pay_delay))
+      {
+        is.default_pay_delay = GNUNET_TIME_UNIT_DAYS;
+      }
+      if (GNUNET_OK !=
+          GNUNET_CONFIGURATION_get_value_time (config,
+                                               "merchant",
+                                               "DEFAULT_REFUND_DELAY",
+                                               &is.default_refund_delay))
+      {
+        is.default_refund_delay = GNUNET_TIME_relative_multiply (
+          GNUNET_TIME_UNIT_DAYS,
+          15);
+      }
+
+      GNUNET_CRYPTO_eddsa_key_create (&merchant_priv.eddsa_priv);
+      GNUNET_CRYPTO_eddsa_key_get_public (&merchant_priv.eddsa_priv,
+                                          &merchant_pub.eddsa_pub);
+      qs = plugin->insert_instance (plugin->cls,
+                                    &merchant_pub,
+                                    &merchant_priv,
+                                    &is,
+                                    &ias,
+                                    false);
+      json_decref (is.address);
+      json_decref (is.jurisdiction);
+      switch (qs)
+      {
+      case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
+        fprintf (stderr,
+                 "`%s' instance created with default settings\n",
+                 instance);
+        break;
+      case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
+        GNUNET_break (0);
+        break;
+      case GNUNET_DB_STATUS_SOFT_ERROR:
+      case GNUNET_DB_STATUS_HARD_ERROR:
+        fprintf (stderr,
+                 "Internal database error.\n");
+        global_ret = 3;
+        break;
+      }
+    }
+    else
+    {
+      fprintf (stderr,
+               "Instance `%s' unknown, cannot reset token\n",
+               instance);
+      global_ret = 2;
+    }
     break;
   case GNUNET_DB_STATUS_SOFT_ERROR:
   case GNUNET_DB_STATUS_HARD_ERROR:
