@@ -225,7 +225,7 @@ run (void *cls,
     TALER_TESTING_cmd_merchant_kyc_get (
       "kyc-auth-done-check",
       merchant_url,
-      NULL, /* default instance */
+      NULL, /* admin instance */
       "instance-create-default-account", /* h_wire_ref: which account to query */
       EXCHANGE_URL,
       TALER_EXCHANGE_KLPT_KYC_AUTH_TRANSFER,
@@ -279,7 +279,7 @@ run (void *cls,
     TALER_TESTING_cmd_merchant_kyc_get (
       "kyc-pending",
       merchant_url,
-      NULL, /* default instance */
+      NULL, /* admin instance */
       "instance-create-default-account", /* h_wire_ref: which account to query */
       EXCHANGE_URL,
       TALER_EXCHANGE_KLPT_KYC_AUTH_TRANSFER,
@@ -531,7 +531,7 @@ run (void *cls,
     TALER_TESTING_cmd_merchant_post_instances (
       "instance-create-default-setup",
       merchant_url,
-      "default",
+      "admin",
       MHD_HTTP_NO_CONTENT),
     TALER_TESTING_cmd_merchant_post_account (
       "instance-create-default-account",
