@@ -38,6 +38,10 @@
 #include <taler/taler_error_codes.h>
 #include "taler_merchant_testing_lib.h"
 
+#ifdef HAVE_DONAU_DONAU_SERVICE_H
+#include <donau/donau_testing_lib.h>
+#endif
+
 
 /**
  * The 'poll-orders-conclude-1' and other 'conclude'
@@ -233,7 +237,7 @@ run (void *cls,
       "4",
       GNUNET_TIME_UNIT_ZERO_TS,
       GNUNET_TIME_UNIT_FOREVER_TS,
-      "CHF:5.0"),
+      "XXX:5.0"),
     TALER_TESTING_cmd_merchant_post_orders_no_claim (
       "create-proposal-4",
       merchant_url,
@@ -494,6 +498,8 @@ run (void *cls,
                                               MHD_HTTP_NO_CONTENT,
                                               "deposit-simple",
                                               NULL),
+    TALER_TESTING_cmd_depositcheck ("run taler-merchant-depositcheck-1",
+                                    config_file),
     TALER_TESTING_cmd_run_tme ("run taler-merchant-reconciliation-1",
                                config_file),
     TALER_TESTING_cmd_merchant_post_transfer2 ("post-transfer-bad",
@@ -1000,7 +1006,7 @@ run (void *cls,
       "instance-create-i1a-auth-ok",
       merchant_url,
       "i1a",
-      RFC_8959_PREFIX "my-secret",
+      "my-secret",
       MHD_HTTP_NO_CONTENT),
     TALER_TESTING_cmd_merchant_get_product ("get-nx-product-i1a-3",
                                             merchant_url_i1a,
@@ -1012,7 +1018,20 @@ run (void *cls,
                                              MHD_HTTP_UNAUTHORIZED,
                                              NULL),
     TALER_TESTING_cmd_set_authorization ("set-auth-valid",
-                                         "Bearer " RFC_8959_PREFIX "my-secret"),
+                                         "Basic aTFhOm15LXNlY3JldA=="),
+    TALER_TESTING_cmd_merchant_post_instance_token (
+      "instance-create-i1a-token-ok",
+      merchant_url,
+      "i1a",
+      "write", /* scope */
+      GNUNET_TIME_UNIT_DAYS, /* duration */
+      GNUNET_YES, /* refreshable */
+      MHD_HTTP_OK),
+    TALER_TESTING_cmd_set_authorization ("unset-auth-valid",
+                                         NULL), // Unset header
+    TALER_TESTING_cmd_merchant_set_instance_token (
+      "instance-create-i1a-token-set",
+      "instance-create-i1a-token-ok"),
     TALER_TESTING_cmd_merchant_get_product ("get-nx-product-i1a-4",
                                             merchant_url_i1a,
                                             "nx-product",
@@ -1022,11 +1041,10 @@ run (void *cls,
                                              merchant_url_i1a,
                                              MHD_HTTP_OK,
                                              NULL),
-    TALER_TESTING_cmd_merchant_post_instance_auth (
-      "instance-create-i1a-change-auth",
-      merchant_url_i1a,
-      NULL,
-      RFC_8959_PREFIX "my-other-secret",
+    TALER_TESTING_cmd_merchant_delete_instance_token (
+      "instance-create-i1a-token-delete",
+      merchant_url,
+      "i1a",
       MHD_HTTP_NO_CONTENT),
     TALER_TESTING_cmd_merchant_get_product ("get-nx-product-i1a-5",
                                             merchant_url_i1a,
@@ -1041,9 +1059,21 @@ run (void *cls,
                                                merchant_url_i1a,
                                                NULL,
                                                MHD_HTTP_UNAUTHORIZED),
-    TALER_TESTING_cmd_set_authorization (
+    TALER_TESTING_cmd_set_authorization ("set-token-auth-valid-again",
+                                         "Basic aTFhOm15LXNlY3JldA=="),
+    TALER_TESTING_cmd_merchant_post_instance_token (
       "set-auth-valid-again",
-      "Bearer " RFC_8959_PREFIX "my-other-secret"),
+      merchant_url,
+      "i1a",
+      "write", /* scope */
+      GNUNET_TIME_UNIT_DAYS, /* duration */
+      GNUNET_YES, /* refreshable */
+      MHD_HTTP_OK),
+    TALER_TESTING_cmd_set_authorization ("unset-auth-valid2",
+                                         NULL), // Unset header
+    TALER_TESTING_cmd_merchant_set_instance_token (
+      "instance-create-i1a-token-set-again",
+      "set-auth-valid-again"),
     TALER_TESTING_cmd_merchant_post_instance_auth (
       "instance-create-i1a-auth-ok-idempotent",
       merchant_url_i1a,
@@ -1668,13 +1698,13 @@ run (void *cls,
      * Move money to the exchange's bank account.
      */
     cmd_transfer_to_exchange ("create-reserve-tokens",
-                              "EUR:10.02"),
+                              "EUR:20.03"),
     /**
      * Make a reserve exist, according to the previous transfer.
      */
     cmd_exec_wirewatch ("wirewatch-1"),
     TALER_TESTING_cmd_check_bank_admin_transfer ("check_bank_transfer-tokens",
-                                                 "EUR:10.02",
+                                                 "EUR:20.03",
                                                  payer_payto,
                                                  exchange_payto,
                                                  "create-reserve-tokens"),
@@ -1688,6 +1718,11 @@ run (void *cls,
                                        "EUR:5",
                                        0,
                                        MHD_HTTP_OK),
+    TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-3",
+                                       "create-reserve-tokens",
+                                       "EUR:5",
+                                       0,
+                                       MHD_HTTP_OK),
     TALER_TESTING_cmd_merchant_post_tokenfamilies (
       "create-upcoming-tokenfamily",
       merchant_url,
@@ -1715,6 +1750,8 @@ run (void *cls,
       merchant_url,
       MHD_HTTP_OK,
       "subscription-upcoming",
+      "A choice in the contract",
+      NULL,
       0,
       1,
       "5-upcoming-output",
@@ -1741,6 +1778,8 @@ run (void *cls,
       merchant_url,
       MHD_HTTP_OK,
       "subscription-1",
+      "A choice in the contract",
+      NULL,
       0,
       1,
       "5-output",
@@ -1764,6 +1803,8 @@ run (void *cls,
       merchant_url,
       MHD_HTTP_OK,
       "subscription-1",
+      "A choice in the contract",
+      NULL,
       1,
       1,
       "5-input-output",
@@ -1797,6 +1838,8 @@ run (void *cls,
       merchant_url,
       MHD_HTTP_OK,
       "subscription-1",
+      "A choice in the contract",
+      NULL,
       1,
       1,
       "5-input-output-2",
@@ -1816,12 +1859,110 @@ run (void *cls,
     TALER_TESTING_cmd_end ()
   };
 
+  #ifdef HAVE_DONAU_DONAU_SERVICE_H
+  const struct DONAU_BearerToken bearer = {
+    .token = NULL
+  };
+
+  struct TALER_TESTING_Command donau[] = {
+    TALER_TESTING_cmd_set_var (
+      "donau",
+      TALER_TESTING_cmd_get_donau ("get-donau",
+                                   cred.cfg,
+                                   true)),
+    TALER_TESTING_cmd_charity_post_merchant ("post-charity",
+                                             "example",
+                                             "example.com",
+                                             "EUR:50", // max_per_year
+                                             &bearer,
+                                             "create-another-order-with-input-and-output",
+                                             // reusing the merchant_reference for merchant_pub
+                                             MHD_HTTP_CREATED),
+    TALER_TESTING_cmd_charity_post_merchant ("post-charity-idempotent",
+                                             "example",
+                                             "example.com",
+                                             "EUR:50", // max_per_year
+                                             &bearer,
+                                             "create-another-order-with-input-and-output",
+                                             // reusing the merchant_reference for merchant_pub
+                                             MHD_HTTP_CREATED),
+    TALER_TESTING_cmd_merchant_post_donau_instance (
+      "post-donau-instance",
+      merchant_url,
+      "create-another-order-with-input-and-output",
+      MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_post_donau_instance (
+      "post-donau-instance-idempotent",
+      merchant_url,
+      "create-another-order-with-input-and-output",
+      MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_get_donau_instances (
+      "get-donau-instances-after-insert",
+      merchant_url,
+      1,
+      MHD_HTTP_OK),
+    TALER_TESTING_cmd_sleep (
+      "In this time donaukeyupdate must fetch the keys from the donau",
+      1),
+    TALER_TESTING_cmd_merchant_get_donau_instances (
+      "get-donau-instance",
+      merchant_url,
+      1,
+      MHD_HTTP_OK),
+    TALER_TESTING_cmd_merchant_post_orders_donau (
+      "create-donau-order",
+      cred.cfg,
+      merchant_url,
+      MHD_HTTP_OK,
+      "donau",
+      GNUNET_TIME_UNIT_ZERO_TS,
+      GNUNET_TIME_UNIT_FOREVER_TS,
+      "EUR:1"),
+    TALER_TESTING_cmd_merchant_pay_order_donau (
+      "pay-donau-order",
+      merchant_url,
+      MHD_HTTP_OK,
+      "create-donau-order",
+      "withdraw-coin-3",
+      "EUR:1", /* full amount */
+      "EUR:0.99", /* amount without fees */
+      "EUR:1", /* donation amount */
+      NULL,
+      0,
+      "post-charity",
+      2025,
+      "7560001010000",
+      "1234"
+      ),
+    TALER_TESTING_cmd_merchant_delete_donau_instance (
+      "delete-donau-instance",
+      merchant_url,
+      1,
+      MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_get_donau_instances (
+      "get-donau-instances-after-delete",
+      merchant_url,
+      0,
+      MHD_HTTP_OK),
+    TALER_TESTING_cmd_end ()
+  };
+  #endif
+
   struct TALER_TESTING_Command commands[] = {
     /* general setup */
     TALER_TESTING_cmd_run_fakebank (
       "run-fakebank",
       cred.cfg,
       "exchange-account-exchange"),
+    #ifdef HAVE_DONAU_DONAU_SERVICE_H
+    TALER_TESTING_cmd_system_start (
+      "start-taler",
+      config_file,
+      "-emaDZ",
+      "-u", "exchange-account-exchange",
+      "-r", "merchant-exchange-test",
+      NULL),
+    #else
     TALER_TESTING_cmd_system_start (
       "start-taler",
       config_file,
@@ -1829,6 +1970,7 @@ run (void *cls,
       "-u", "exchange-account-exchange",
       "-r", "merchant-exchange-test",
       NULL),
+    #endif
     TALER_TESTING_cmd_get_exchange (
       "get-exchange",
       cred.cfg,
@@ -2141,6 +2283,25 @@ run (void *cls,
                              repurchase),
     TALER_TESTING_cmd_batch ("tokens",
                              tokens),
+
+    #ifdef HAVE_DONAU_DONAU_SERVICE_H
+    TALER_TESTING_cmd_batch ("donau",
+                             donau),
+    #endif
+
+    TALER_TESTING_cmd_merchant_get_statisticsamount ("stats-refund",
+                                                     merchant_url,
+                                                     "refunds-granted", 6, 0,
+                                                     MHD_HTTP_OK),
+    TALER_TESTING_cmd_merchant_get_statisticscounter ("stats-tokens-issued",
+                                                      merchant_url,
+                                                      "tokens-issued", 6, 0,
+                                                      MHD_HTTP_OK),
+    TALER_TESTING_cmd_merchant_get_statisticscounter ("stats-tokens-used",
+                                                      merchant_url,
+                                                      "tokens-used", 6, 0,
+                                                      MHD_HTTP_OK),
+
     /**
      * End the suite.
      */
