@@ -45,7 +45,7 @@ else
 
     ACCOUNT="exchange-account-1"
     WIRE_METHOD="iban"
-    BANK_FLAGS="-ns -d $WIRE_METHOD -u $ACCOUNT"
+    BANK_FLAGS="-n -d $WIRE_METHOD -u $ACCOUNT"
     BANK_URL="http://localhost:18082/"
 fi
 
@@ -298,7 +298,7 @@ fi
 WIRE_DEADLINE=$(jq -r .contract_terms.wire_transfer_deadline.t_s < "$LAST_RESPONSE")
 NOW=$(date +%s)
 
-TO_SLEEP="$(( 1 + WIRE_DEADLINE - NOW ))"
+TO_SLEEP="$(( 3600 + WIRE_DEADLINE - NOW ))"
 echo -n "Perform wire transfers (with ${TO_SLEEP}s timeshift) ..."
 taler-exchange-aggregator \
     -y \
@@ -331,6 +331,23 @@ taler-merchant-wirewatch \
     -L INFO &> merchant-wirewatch.log
 echo " OK"
 
+echo -n "Obtaining deposit data from exchange..."
+taler-merchant-depositcheck \
+    -c "$CONF" \
+    -e "http://localhost:8081/" \
+    -T "${TO_SLEEP}000000" \
+    -t \
+    -L INFO &> merchant-depositcheck.log
+echo " OK"
+
+echo -n "Obtaining reconciliation data from exchange..."
+taler-merchant-reconciliation \
+    -c "$CONF" \
+    -T "${TO_SLEEP}000000" \
+    -t \
+    -L INFO &> merchant-reconciliation.log
+echo " OK"
+
 echo -n "Fetching wire transfers of ADMIN instance ..."
 STATUS=$(curl 'http://localhost:9966/private/transfers' \
               -w "%{http_code}" \
@@ -347,7 +364,7 @@ then
 fi
 echo " OK"
 
-echo -n "Integrating wire transfer data with exchange..."
+echo -n "Reconciling wire transfer data with exchange..."
 taler-merchant-reconciliation \
     -c "$CONF" \
     -t \
