@@ -120,11 +120,12 @@ delete_product_run (void *cls,
   struct DeleteProductState *dis = cls;
 
   dis->is = is;
-  dis->pdh = TALER_MERCHANT_product_delete (is->ctx,
-                                            dis->merchant_url,
-                                            dis->product_id,
-                                            &delete_product_cb,
-                                            dis);
+  dis->pdh = TALER_MERCHANT_product_delete (
+    TALER_TESTING_interpreter_get_context (is),
+    dis->merchant_url,
+    dis->product_id,
+    &delete_product_cb,
+    dis);
   GNUNET_assert (NULL != dis->pdh);
 }
 
