@@ -77,9 +77,10 @@ delete_template_cb (void *cls,
   dis->tdh = NULL;
   if (dis->http_status != hr->http_status)
   {
-    TALER_TESTING_unexpected_status (dis->is,
-                                     hr->http_status,
-                                     dis->http_status);
+    TALER_TESTING_unexpected_status_with_body (dis->is,
+                                               hr->http_status,
+                                               dis->http_status,
+                                               hr->reply);
     return;
   }
   switch (hr->http_status)
