@@ -163,6 +163,8 @@ patch_instance_run (void *cls,
     pis->use_stefan,
     pis->default_wire_transfer_delay,
     pis->default_pay_delay,
+    GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_DAYS,
+                                   15),
     &patch_instance_cb,
     pis);
   GNUNET_assert (NULL != pis->iph);
