@@ -172,21 +172,22 @@ patch_product_run (void *cls,
   struct PatchProductState *pis = cls;
 
   pis->is = is;
-  pis->iph = TALER_MERCHANT_product_patch (is->ctx,
-                                           pis->merchant_url,
-                                           pis->product_id,
-                                           pis->description,
-                                           pis->description_i18n,
-                                           pis->unit,
-                                           &pis->price,
-                                           pis->image,
-                                           pis->taxes,
-                                           pis->total_stock,
-                                           pis->total_lost,
-                                           pis->address,
-                                           pis->next_restock,
-                                           &patch_product_cb,
-                                           pis);
+  pis->iph = TALER_MERCHANT_product_patch (
+    TALER_TESTING_interpreter_get_context (is),
+    pis->merchant_url,
+    pis->product_id,
+    pis->description,
+    pis->description_i18n,
+    pis->unit,
+    &pis->price,
+    pis->image,
+    pis->taxes,
+    pis->total_stock,
+    pis->total_lost,
+    pis->address,
+    pis->next_restock,
+    &patch_product_cb,
+    pis);
   GNUNET_assert (NULL != pis->iph);
 }
 
@@ -209,18 +210,17 @@ patch_product_traits (void *cls,
 {
   struct PatchProductState *pps = cls;
   struct TALER_TESTING_Trait traits[] = {
-    TALER_TESTING_make_trait_product_description (&pps->description),
+    TALER_TESTING_make_trait_product_description (pps->description),
     TALER_TESTING_make_trait_i18n_description (pps->description_i18n),
-    TALER_TESTING_make_trait_product_unit (&pps->unit),
+    TALER_TESTING_make_trait_product_unit (pps->unit),
     TALER_TESTING_make_trait_amount (&pps->price),
-    TALER_TESTING_make_trait_product_image (
-      (const char **) &pps->image),
+    TALER_TESTING_make_trait_product_image (pps->image),
     TALER_TESTING_make_trait_taxes (pps->taxes),
     TALER_TESTING_make_trait_product_stock (&pps->total_stock),
     TALER_TESTING_make_trait_address (pps->address),
     TALER_TESTING_make_trait_timestamp (0,
                                         &pps->next_restock),
-    TALER_TESTING_make_trait_product_id (&pps->product_id),
+    TALER_TESTING_make_trait_product_id (pps->product_id),
     TALER_TESTING_trait_end (),
   };
 
