@@ -172,6 +172,8 @@ post_instances_run (
     pis->use_stefan,
     pis->default_wire_transfer_delay,
     pis->default_pay_delay,
+    GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_DAYS,
+                                   15),
     pis->auth_token,
     &post_instances_cb,
     pis);
